//------------------------------------------------------
// module  : FilteredRetroProjection.cc
// auteur  : Mignotte Max
// date    : 
// version : 1.0
// langage : C++
// labo    :
//------------------------------------------------------
//  

//------------------------------------------------
// FICHIERS INCLUS -------------------------------
//------------------------------------------------
#include "ImagePGM.h"
#include "Tomograph.h"
#include "FilteredRetroProjection.h"

extern "C" {
#include "Graph_LnxExtern.h"
}

//------------------------------------------------
// DEFINITIONS -----------------------------------
//------------------------------------------------
#define NAME_IMG_TFRADON  "TransfRadon"


//------------------------------------------------
// CONSTANTES ------------------------------------
//------------------------------------------------
const int ZOOM=1;
const int QUIT=0;
const int SAVE=0;

const int NBSLICES=180;
const int FILTRAGE=0;
const float ALPHA=1.0;

//------------------------------------------------
// VARIABLES GLOBALES ----------------------------
//------------------------------------------------
char Name_img[NCHAR];

int nbslices;
int Filtrage;
float Alpha;

int zoom;
int flag_save;
int flag_quit;


//---------------------------------------------------------
// Enregistre les arguments de la ligne de commande      
//---------------------------------------------------------
void read_arguments(int argc,char** argv)
{
   int i;

   //Options
  if (argc<2) 
    { printf("\n Usage FilteredRetroProjection [Img]"); 
      printf("\n Les options sont : valeur par defaut indique dans ()");
      printf("\n");
      printf("\n -n Nb. Tranches");
      printf("\n -------------");
      printf("\n -f Type Filtrage : RampFilter:0  RaisedCosineRolloff:1   Default:(%d)",Filtrage);
      printf("\n -a Alpha (%f)",Alpha);
      printf("\n -------------");
      printf("\n -z Zoom (%d)",ZOOM);
      printf("\n -q Quit (Sans)"); 
      printf("\n\n");
      exit(-1); }

      //Chargement Fichier
      strcpy(Name_img,argv[1]);      

   //Boucle
   for(i=2;i<argc;i++)
     {   
          switch(argv[i][1])
          {
           case 'n': nbslices=atoi(argv[++i]);     break;
           case 'f': Filtrage=atoi(argv[++i]);     break;
           case 'a': Alpha=atof(argv[++i]);        break;
           case 's': flag_save=1;                  break; 
           case 'q': flag_quit=1;                  break;
           case 'z': zoom=atoi(argv[++i]);         break;
           case '?':                               break;                
          }    
     } 
 }

//----------------------------------------------------------
//----------------------------------------------------------
// PROGRAMME PRINCIPAL -------------------------------------
//----------------------------------------------------------
//----------------------------------------------------------
int main(int argc,char** argv)
{
 int   flag;

 //Pour Xwindow
 //------------
 XEvent ev;
 Window win_ppicture;
 Window win_ppicture_TransfRadon;
 Window win_ppicture_TransfRadonMFFT;
 Window win_ppicture_reconst;
 XImage *x_ppicture; 
 XImage *x_ppicture_TransfRadon;
 XImage *x_ppicture_TransfRadonMFFT; 
 XImage *x_ppicture_reconst;
 
 char nomfen_ppicture[NCHAR];
 char nomfen_ppicture_TransfRadon[NCHAR];
 char nomfen_ppicture_TransfRadonMFFT[NCHAR]; 
 char nomfen_ppicture_reconst[NCHAR]; 

 //Initialisation
 //--------------
 Alpha=ALPHA;
 Filtrage=FILTRAGE;
 nbslices=NBSLICES;
 zoom=ZOOM;
 flag_save=SAVE;
 flag_quit=QUIT;
 flag=0;

 //Lis Arguments
 //-------------
 read_arguments(argc,argv);

 //Affiche options
 //---------------
 printf("\n");
 printf("\n\n Info: ");
 printf("\n -----");
 printf("\n Ouverture de l'image : <%s>",Name_img);
 printf("\n -----");
 printf("\n Nb. Slices : %d",nbslices);
 printf("\n Type Filtrage : %d",Filtrage);
 printf("\n Alpha : %.2f",Alpha);

 printf("\n Zoom: %d",zoom); 
 if (flag_save)   printf("\n -> Avec Sauvegarde");
 if (flag_quit)   printf("\n -> Avec Quit");

//-------------------------------
//---- Chargement des Images -------------------------------
//-------------------------------

 //Definitions
 //-----------
 ImagePGM* img;

 //Recuperation Images & Info
 //--------------------------
 img=new ImagePGM(Name_img);
 
 //Information 
 printf("\n Taille de l'image: [%d - %d]",(*img).length,(*img).width);

  //-----------------------------------
  //---- Algorithme Tomograph ---------
  //-----------------------------------
  Tomograph tomo((*img).length,(*img).width,nbslices);
  tomo.SimulateTomograph((*img).datai);

  //-------------------------------------
  //--- Reconstruction RetProj. Filtre --
  //-------------------------------------
  FilteredRetroProjection retrproj((*img).length,(*img).width,nbslices);
  retrproj.Options((*img).datai);
  retrproj.Options2(Filtrage,Alpha);
  retrproj.Reconstruction(tomo.mat_radonRFFT,tomo.mat_radonIFFT);


//---------------------------------------------------------------
//---------------- visu sous XWINDOW ----------------------------
//---------------------------------------------------------------
 if (!flag_quit)
 {
 //ouverture session graphique
 if (open_display()<0) printf(" Impossible d'ouvrir une session graphique");

 //commentaire dans les photos affichees
 sprintf(nomfen_ppicture,"Image : %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadon,"TransfRadon : %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadonMFFT,"Module FFT (TransfRadon : %s)",Name_img); 
 sprintf(nomfen_ppicture_reconst,"Image : %s",Name_img);
 
 //creation des fenetres
 win_ppicture=fabrique_window(nomfen_ppicture,10,10,(*img).width,(*img).length,zoom);
 win_ppicture_TransfRadon=fabrique_window(nomfen_ppicture_TransfRadon,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_TransfRadonMFFT=fabrique_window(nomfen_ppicture_TransfRadonMFFT,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_reconst=fabrique_window(nomfen_ppicture_reconst,10,10,(*img).width,(*img).length,zoom);
 
 //creation des Ximages
 x_ppicture=cree_Ximage4((*img).datai,zoom,(*img).length,(*img).width);
 x_ppicture_TransfRadon=cree_Ximage2(tomo.mat_radon_visu,zoom,tomo.length_radon,tomo.width_radon);
 x_ppicture_TransfRadonMFFT=cree_Ximage2(tomo.mat_radonMFFT,zoom,tomo.length_radon,tomo.width_radon);
 x_ppicture_reconst=cree_Ximage2(retrproj.mat_reconst,zoom,retrproj.length,retrproj.width);

 printf("\n\n Pour quitter,appuyer sur la barre d'espace");

 //boucle d'evenements
  for(;;)
     {
      XNextEvent(display,&ev);
       switch(ev.type)
        {
         case Expose:
         XPutImage(display,win_ppicture,gc,x_ppicture,0,0,0,0,
                   x_ppicture->width,x_ppicture->height);
         XPutImage(display,win_ppicture_TransfRadon,gc,x_ppicture_TransfRadon,0,0,0,0,
                   x_ppicture_TransfRadon->width,x_ppicture_TransfRadon->height);
         XPutImage(display,win_ppicture_TransfRadonMFFT,gc,x_ppicture_TransfRadonMFFT,0,0,0,0,
                   x_ppicture_TransfRadonMFFT->width,x_ppicture_TransfRadonMFFT->height);
         XPutImage(display,win_ppicture_reconst,gc,x_ppicture_reconst,0,0,0,0,
                   x_ppicture_reconst->width,x_ppicture_reconst->height);
         break;

         case KeyPress:
         XDestroyImage(x_ppicture);
         XDestroyImage(x_ppicture_TransfRadon);
         XDestroyImage(x_ppicture_TransfRadonMFFT); 
          XDestroyImage(x_ppicture_reconst); 

         XFreeGC(display,gc);
         XCloseDisplay(display);
         flag=1;
         break;
         }
   if (flag==1) break;
   }
 } 
       
//--------------- fin de la session graphique --------------     
//----------------------------------------------------------
 
   //retour sans probleme
   //--------------------
   cout << "\n\n\n C'est fini... \n\n\n\n\n" << endl ;
   return 0;
 }
 


