//------------------------------------------------------
// module  : FilteredRetroProjection.cc
// auteur  : Mignotte Max
// date    : 
// version : 1.0
// langage : C++
// labo    :
//------------------------------------------------------
//  

//------------------------------------------------
// FICHIERS INCLUS -------------------------------
//------------------------------------------------
#include "Tomograph.h"
#include "FilteredRetroProjection.h"


//------------------------------------------------
// DEFINITIONS -----------------------------------
//------------------------------------------------
#define NAME_IMG_TFRADON  "TransfRadon"


//------------------------------------------------
// CONSTANTES ------------------------------------
//------------------------------------------------
const int ZOOM=1;
const int QUIT=0;
const int SAVE=0;

const int NBSLICES=180;
const int FILTRAGE=0;
const float ALPHA=1.0;

const int NCHAR2=200;

//------------------------------------------------
// VARIABLES GLOBALES ----------------------------
//------------------------------------------------
char Name_img[NCHAR2];

int nbslices;
int Filtrage;
float Alpha;

int zoom;
int flag_save;
int flag_quit;

//---------------------------------------------------------------------------------
//--- Windows ---------------------------------------------------------------------
//---------------------------------------------------------------------------------
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>

Display   *display;
int	  screen_num;
int 	  depth;
Window	  root;
Visual*	  visual;
GC	  gc;

/************************************************************************/
/* OPEN_DISPLAY()							*/
/************************************************************************/
int open_display()
{
  if ((display=XOpenDisplay(NULL))==NULL)
   { printf("Connection impossible\n");
     return(-1); }

  else
   { screen_num=DefaultScreen(display);
     visual=DefaultVisual(display,screen_num);
     depth=DefaultDepth(display,screen_num);
     root=RootWindow(display,screen_num);
     return 0; }
}

/************************************************************************/
/* FABRIQUE_WINDOW()							*/
/* Cette fonction cre une fenetre X et l'affiche  l'cran.	        */
/************************************************************************/
Window fabrique_window(char *nom_fen,int x,int y,int width,int height,int zoom)
{
  Window                 win;
  XSizeHints      size_hints;
  XWMHints          wm_hints;
  XClassHint     class_hints;
  XTextProperty  windowName, iconName;

  char *name=nom_fen;

  if(zoom<0) { width/=-zoom; height/=-zoom; }
  if(zoom>0) { width*=zoom;  height*=zoom;  }

  win=XCreateSimpleWindow(display,root,x,y,width,height,1,0,255);

  size_hints.flags=PPosition|PSize|PMinSize;
  size_hints.min_width=width;
  size_hints.min_height=height;

  XStringListToTextProperty(&name,1,&windowName);
  XStringListToTextProperty(&name,1,&iconName);
  wm_hints.initial_state=NormalState;
  wm_hints.input=True;
  wm_hints.flags=StateHint|InputHint;
  class_hints.res_name=nom_fen;
  class_hints.res_class=nom_fen;

  XSetWMProperties(display,win,&windowName,&iconName,
                   NULL,0,&size_hints,&wm_hints,&class_hints);

  gc=XCreateGC(display,win,0,NULL);

  XSelectInput(display,win,ExposureMask|KeyPressMask|ButtonPressMask| 
               ButtonReleaseMask|ButtonMotionMask|PointerMotionHintMask| 
               StructureNotifyMask);

  XMapWindow(display,win);
  return(win);
}

/****************************************************************************/
/* CREE_XIMAGE2()							    */
/* Cre une XImage  partir d'un tableau de float                           */
/* L'image peut subir un zoom.						    */
/****************************************************************************/
XImage* cree_Ximage2(float** mat,int z,int length,int width)
{
  int lgth,wdth,lig,col,zoom_col,zoom_lig;
  float somme;
  unsigned char	 pix;
  unsigned char* dat;
  XImage* imageX;

  /*Zoom positiv*/
  /*------------*/
  if (z>0)
  {
   lgth=length*z;
   wdth=width*z;

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<lgth;lig=lig+z) for(col=0;col<wdth;col=col+z)
   { 
    pix=(unsigned char)mat[lig/z][col/z];
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
      { 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+0)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+1)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+2)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+3)]=pix; 
       }
    }
  } /*--------------------------------------------------------*/

  /*Zoom negatifv*/
  /*------------*/
  else
  {
   z=-z;
   lgth=(length/z);
   wdth=(width/z);

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<(lgth*z);lig=lig+z) for(col=0;col<(wdth*z);col=col+z)
   {  
    somme=0.0;
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
     somme+=mat[lig+zoom_lig][col+zoom_col];
           
     somme/=(z*z);    
     dat[((lig/z)*wdth*4)+((4*(col/z))+0)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+1)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+2)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+3)]=(unsigned char)somme; 
   }
  } /*--------------------------------------------------------*/

  imageX=XCreateImage(display,visual,depth,ZPixmap,0,(char*)dat,wdth,lgth,16,wdth*4);
  return (imageX);
}

/****************************************************************************/
/* CREE_XIMAGE4()							    */
/* Cre une XImage  partir d'un tableau d'entier                           */
/* L'image peut subir un zoom.						    */
/****************************************************************************/
XImage* cree_Ximage4(int** mat,int z,int length,int width)
{
  int lgth,wdth,lig,col,zoom_col,zoom_lig;
  float somme;
  unsigned char	 pix;
  unsigned char* dat;
  XImage* imageX;

  /*Zoom positif*/
  /*------------*/
  if (z>0)
  {
   lgth=length*z;
   wdth=width*z;

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<lgth;lig=lig+z) for(col=0;col<wdth;col=col+z)
   { 
    pix=(unsigned char)mat[lig/z][col/z];
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
      { 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+0)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+1)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+2)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+3)]=pix; 
       }
    }
  } /*--------------------------------------------------------*/

  /*Zoom negatif*/
  /*------------*/
  else
  {
   z=-z;
   lgth=(length/z);
   wdth=(width/z);

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<(lgth*z);lig=lig+z) for(col=0;col<(wdth*z);col=col+z)
   {  
    somme=0.0;
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
     somme+=mat[lig+zoom_lig][col+zoom_col];
           
     somme/=(z*z);    
     dat[((lig/z)*wdth*4)+((4*(col/z))+0)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+1)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+2)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+3)]=(unsigned char)somme; 
   }
  } /*--------------------------------------------------------*/

  imageX=XCreateImage(display,visual,depth,ZPixmap,0,(char*)dat,wdth,lgth,16,wdth*4);
  return (imageX);
}

//----End Windows---------------------------------------------------------
//------------------------------------------------------------------------


//---------------------------------------------------------
// Enregistre les arguments de la ligne de commande      
//---------------------------------------------------------
void read_arguments(int argc,char** argv)
{
   int i;

   //Options
  if (argc<2) 
    { printf("\n Usage FilteredRetroProjection [Img]"); 
      printf("\n Les options sont : valeur par defaut indique dans ()");
      printf("\n");
      printf("\n -n Nb. Tranches");
      printf("\n -------------");
      printf("\n -f Type Filtrage : RampFilter:0  RaisedCosineRolloff:1   Default:(%d)",Filtrage);
      printf("\n -a Alpha (%f)",Alpha);
      printf("\n -------------");
      printf("\n -z Zoom (%d)",ZOOM);
      printf("\n -q Quit (Sans)"); 
      printf("\n\n");
      exit(-1); }

      //Chargement Fichier
      strcpy(Name_img,argv[1]);      

   //Boucle
   for(i=2;i<argc;i++)
     {   
          switch(argv[i][1])
          {
           case 'n': nbslices=atoi(argv[++i]);     break;
           case 'f': Filtrage=atoi(argv[++i]);     break;
           case 'a': Alpha=atof(argv[++i]);        break;
           case 's': flag_save=1;                  break; 
           case 'q': flag_quit=1;                  break;
           case 'z': zoom=atoi(argv[++i]);         break;
           case '?':                               break;                
          }    
     } 
 }

//----------------------------------------------------------
//----------------------------------------------------------
// PROGRAMME PRINCIPAL -------------------------------------
//----------------------------------------------------------
//----------------------------------------------------------
int main(int argc,char** argv)
{
 int   flag;
 int length,width;

 //Pour Xwindow
 //------------
 XEvent ev;
 Window win_ppicture;
 Window win_ppicture_TransfRadon;
 Window win_ppicture_TransfRadonMFFT;
 Window win_ppicture_reconst;
 XImage *x_ppicture; 
 XImage *x_ppicture_TransfRadon;
 XImage *x_ppicture_TransfRadonMFFT; 
 XImage *x_ppicture_reconst;
 
 char nomfen_ppicture[NCHAR2];
 char nomfen_ppicture_TransfRadon[NCHAR2];
 char nomfen_ppicture_TransfRadonMFFT[NCHAR2]; 
 char nomfen_ppicture_reconst[NCHAR2]; 

 //Initialisation
 //--------------
 Alpha=ALPHA;
 Filtrage=FILTRAGE;
 nbslices=NBSLICES;
 zoom=ZOOM;
 flag_save=SAVE;
 flag_quit=QUIT;
 flag=0;

 //Lis Arguments
 //-------------
 read_arguments(argc,argv);

 //Affiche options
 //---------------
 printf("\n");
 printf("\n\n Info: ");
 printf("\n -----");
 printf("\n Ouverture de l'image : <%s>",Name_img);
 printf("\n -----");
 printf("\n Nb. Slices : %d",nbslices);
 printf("\n Type Filtrage : %d",Filtrage);
 printf("\n Alpha : %.2f",Alpha);

 printf("\n Zoom: %d",zoom); 
 if (flag_save)   printf("\n -> Avec Sauvegarde");
 if (flag_quit)   printf("\n -> Avec Quit");

//-------------------------------
//---- Chargement des Images -------------------------------
//-------------------------------

  //Recuperation Images & Info
  //--------------------------
  GetLengthWidth(Name_img,&length,&width);
  int** datai=imatrix_allocate_2d(length,width);
  load_image_pgm(Name_img,datai,length,width);
 
  //Information 
  printf("\n Taille de l'image: [%d - %d]",length,width);
  fflush(stdout); 



  //-----------------------------------
  //---- Algorithme Tomograph ---------
  //-----------------------------------
  Tomograph tomo(length,width,nbslices);
  tomo.SimulateTomograph(datai);

  //-------------------------------------
  //--- Reconstruction RetProj. Filtre --
  //-------------------------------------
  FilteredRetroProjection retrproj(length,width,nbslices);
  retrproj.Options(datai);
  retrproj.Filtrage=Filtrage;
  retrproj.Alpha=Alpha;
  retrproj.Reconstruction(tomo.mat_radonRFFT,tomo.mat_radonIFFT);

  //-------------------------------------
  //--- Recal
  //-------------------------------------
  Recal(retrproj.mat_reconst,length,width);


//---------------------------------------------------------------
//---------------- visu sous XWINDOW ----------------------------
//---------------------------------------------------------------
 if (!flag_quit)
 {
 //ouverture session graphique
 if (open_display()<0) printf(" Impossible d'ouvrir une session graphique");

 //commentaire dans les photos affichees
 sprintf(nomfen_ppicture,"Image : %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadon,"TransfRadon : %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadonMFFT,"Module FFT (TransfRadon : %s)",Name_img); 
 sprintf(nomfen_ppicture_reconst,"Image : %s",Name_img);
 
 //creation des fenetres
 win_ppicture=fabrique_window(nomfen_ppicture,10,10,width,length,zoom);
 win_ppicture_TransfRadon=fabrique_window(nomfen_ppicture_TransfRadon,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_TransfRadonMFFT=fabrique_window(nomfen_ppicture_TransfRadonMFFT,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_reconst=fabrique_window(nomfen_ppicture_reconst,10,10,width,length,zoom);
 
 //creation des Ximages
 x_ppicture=cree_Ximage4(datai,zoom,length,width);
 x_ppicture_TransfRadon=cree_Ximage2(tomo.mat_radon_visu,zoom,tomo.length_radon,tomo.width_radon);
 x_ppicture_TransfRadonMFFT=cree_Ximage2(tomo.mat_radonMFFT,zoom,tomo.length_radon,tomo.width_radon);
 x_ppicture_reconst=cree_Ximage2(retrproj.mat_reconst,zoom,retrproj.length,retrproj.width);

 printf("\n\n Pour quitter,appuyer sur la barre d'espace");

 //boucle d'evenements
  for(;;)
     {
      XNextEvent(display,&ev);
       switch(ev.type)
        {
         case Expose:
         XPutImage(display,win_ppicture,gc,x_ppicture,0,0,0,0,
                   x_ppicture->width,x_ppicture->height);
         XPutImage(display,win_ppicture_TransfRadon,gc,x_ppicture_TransfRadon,0,0,0,0,
                   x_ppicture_TransfRadon->width,x_ppicture_TransfRadon->height);
         XPutImage(display,win_ppicture_TransfRadonMFFT,gc,x_ppicture_TransfRadonMFFT,0,0,0,0,
                   x_ppicture_TransfRadonMFFT->width,x_ppicture_TransfRadonMFFT->height);
         XPutImage(display,win_ppicture_reconst,gc,x_ppicture_reconst,0,0,0,0,
                   x_ppicture_reconst->width,x_ppicture_reconst->height);
         break;

         case KeyPress:
         XDestroyImage(x_ppicture);
         XDestroyImage(x_ppicture_TransfRadon);
         XDestroyImage(x_ppicture_TransfRadonMFFT); 
          XDestroyImage(x_ppicture_reconst); 

         XFreeGC(display,gc);
         XCloseDisplay(display);
         flag=1;
         break;
         }
   if (flag==1) break;
   }
 } 
       
//--------------- fin de la session graphique --------------     
//----------------------------------------------------------
 
   //retour sans probleme
   //--------------------
   cout << "\n\n\n C'est fini... \n\n\n\n\n" << endl ;
   return 0;
 }
 


