//------------------------------------------------------
// module  : ICD.cc
// auteur  : Mignotte Max
// date    : 
// version : 1.0
// langage : C++
// labo    :
//------------------------------------------------------
//  

//------------------------------------------------
// FICHIERS INCLUS -------------------------------
//------------------------------------------------
#include "ICD.h"

//------------------------------------------------
// CONSTANTES ------------------------------------
//------------------------------------------------
const int ZOOM=1;
const int QUIT=0;
const int SAVE=0;
const int TAB=0;
const int TEST=0;
const int SPECT=0;
const int INIT=0;

const int NBSLICES=180;
const int DVARIANCE=0;
const int NBITERS=50;
const int METHOD=0;
const int SUBMETHOD=0;

const float GAMMA=0.00;   
const float QPRIOR=2.0;

const float VARNOISE=0.0;
const float SCALEREDUC=1.0;
const int   NB_GREYLEVELS=256;
const int   WIDTH_BLUR=0;
const float FACTATTEN=1.0;
const float VALUE_HYPERPAR=6.0;

const char COMMENT[10]="";

//------------------------------------------------
// VARIABLES GLOBALES ----------------------------
//------------------------------------------------
char Name_img[NCHAR];
char Name_spect[NCHAR];
char Name_spect2[NCHAR];

int    Method;
int    SubMethod;

int    nbslices;
int    init;
int    Dvariance;
int    NbIters;
float  Gamma;
float  Qprior;
float  hyper;

float  scalereduc;
int    NbGreyLevels;
float  varnoise;
int    WidthBlur;
float  factatten;
int    slice_number;
float  rcf_factor;
int    cutoff;
int    cutoff2;

int zoom;
int flag_save;
int flag_quit;
int flag_tab;
int flag_test;
int flag_spect;

char Comment[10];

//---------------------------------------------------------------------------------
//--- Windows ---------------------------------------------------------------------
//---------------------------------------------------------------------------------
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>

Display   *display;
int	  screen_num;
int 	  depth;
Window	  root;
Visual*	  visual;
GC	  gc;

/************************************************************************/
/* OPEN_DISPLAY()							*/
/************************************************************************/
int open_display()
{
  if ((display=XOpenDisplay(NULL))==NULL)
   { printf("Connection impossible\n");
     return(-1); }

  else
   { screen_num=DefaultScreen(display);
     visual=DefaultVisual(display,screen_num);
     depth=DefaultDepth(display,screen_num);
     root=RootWindow(display,screen_num);
     return 0; }
}

/************************************************************************/
/* FABRIQUE_WINDOW()							*/
/* Cette fonction cre une fenetre X et l'affiche  l'cran.	        */
/************************************************************************/
Window fabrique_window(char *nom_fen,int x,int y,int width,int height,int zoom)
{
  Window                 win;
  XSizeHints      size_hints;
  XWMHints          wm_hints;
  XClassHint     class_hints;
  XTextProperty  windowName, iconName;

  char *name=nom_fen;

  if(zoom<0) { width/=-zoom; height/=-zoom; }
  if(zoom>0) { width*=zoom;  height*=zoom;  }

  win=XCreateSimpleWindow(display,root,x,y,width,height,1,0,255);

  size_hints.flags=PPosition|PSize|PMinSize;
  size_hints.min_width=width;
  size_hints.min_height=height;

  XStringListToTextProperty(&name,1,&windowName);
  XStringListToTextProperty(&name,1,&iconName);
  wm_hints.initial_state=NormalState;
  wm_hints.input=True;
  wm_hints.flags=StateHint|InputHint;
  class_hints.res_name=nom_fen;
  class_hints.res_class=nom_fen;

  XSetWMProperties(display,win,&windowName,&iconName,
                   NULL,0,&size_hints,&wm_hints,&class_hints);

  gc=XCreateGC(display,win,0,NULL);

  XSelectInput(display,win,ExposureMask|KeyPressMask|ButtonPressMask| 
               ButtonReleaseMask|ButtonMotionMask|PointerMotionHintMask| 
               StructureNotifyMask);

  XMapWindow(display,win);
  return(win);
}

/****************************************************************************/
/* CREE_XIMAGE2()							    */
/* Cre une XImage  partir d'un tableau de float                           */
/* L'image peut subir un zoom.						    */
/****************************************************************************/
XImage* cree_Ximage2(float** mat,int z,int length,int width)
{
  int lgth,wdth,lig,col,zoom_col,zoom_lig;
  float somme;
  unsigned char	 pix;
  unsigned char* dat;
  XImage* imageX;

  /*Zoom positiv*/
  /*------------*/
  if (z>0)
  {
   lgth=length*z;
   wdth=width*z;

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<lgth;lig=lig+z) for(col=0;col<wdth;col=col+z)
   { 
    pix=(unsigned char)mat[lig/z][col/z];
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
      { 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+0)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+1)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+2)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+3)]=pix; 
       }
    }
  } /*--------------------------------------------------------*/

  /*Zoom negatifv*/
  /*------------*/
  else
  {
   z=-z;
   lgth=(length/z);
   wdth=(width/z);

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<(lgth*z);lig=lig+z) for(col=0;col<(wdth*z);col=col+z)
   {  
    somme=0.0;
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
     somme+=mat[lig+zoom_lig][col+zoom_col];
           
     somme/=(z*z);    
     dat[((lig/z)*wdth*4)+((4*(col/z))+0)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+1)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+2)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+3)]=(unsigned char)somme; 
   }
  } /*--------------------------------------------------------*/

  imageX=XCreateImage(display,visual,depth,ZPixmap,0,(char*)dat,wdth,lgth,16,wdth*4);
  return (imageX);
}

/****************************************************************************/
/* CREE_XIMAGE4()							    */
/* Cre une XImage  partir d'un tableau d'entier                           */
/* L'image peut subir un zoom.						    */
/****************************************************************************/
XImage* cree_Ximage4(int** mat,int z,int length,int width)
{
  int lgth,wdth,lig,col,zoom_col,zoom_lig;
  float somme;
  unsigned char	 pix;
  unsigned char* dat;
  XImage* imageX;

  /*Zoom positif*/
  /*------------*/
  if (z>0)
  {
   lgth=length*z;
   wdth=width*z;

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<lgth;lig=lig+z) for(col=0;col<wdth;col=col+z)
   { 
    pix=(unsigned char)mat[lig/z][col/z];
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
      { 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+0)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+1)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+2)]=pix;
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+3)]=pix; 
       }
    }
  } /*--------------------------------------------------------*/

  /*Zoom negatif*/
  /*------------*/
  else
  {
   z=-z;
   lgth=(length/z);
   wdth=(width/z);

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<(lgth*z);lig=lig+z) for(col=0;col<(wdth*z);col=col+z)
   {  
    somme=0.0;
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
     somme+=mat[lig+zoom_lig][col+zoom_col];
           
     somme/=(z*z);    
     dat[((lig/z)*wdth*4)+((4*(col/z))+0)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+1)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+2)]=(unsigned char)somme;
     dat[((lig/z)*wdth*4)+((4*(col/z))+3)]=(unsigned char)somme; 
   }
  } /*--------------------------------------------------------*/

  imageX=XCreateImage(display,visual,depth,ZPixmap,0,(char*)dat,wdth,lgth,16,wdth*4);
  return (imageX);
}

//----End Windows---------------------------------------------------------
//------------------------------------------------------------------------

//---------------------------------------------------------
// Enregistre les arguments de la ligne de commande      
//---------------------------------------------------------
void read_arguments(int argc,char** argv)
{
   int i;

   //Options
  if (argc<2) 
    { printf("\n --------------------------------------------");
      printf("\n Usage ICD [Img]"); 
      printf("\n Options are : default value indicated in ()");
      printf("\n");
      printf("\n ---[Initializations]-------");  
      printf("\n -i ? Initialization  (default::%d)",INIT);
      printf("\n        > FBP:[0]   Filtered Back Projection"); 
      printf("\n        > CST:[2]   Constant");
      printf("\n");
      printf("\n ---[Methods]-------------");
      printf("\n -m ? Methods         (Default::%d)",Method);
      printf("\n        > [-1]  Init.");
      printf("\n          --------");
      printf("\n        > [-13] Penalized EM");
      printf("\n          >>> Tests-----");
      printf("\n                        M #  >  -3  PML with Butterworth filter at each restoration > Best ML");
      printf("\n                        M #  >  -2  PML with Butterworth filter > end iter");
      printf("\n                        M #  >  -1  PML");
      printf("\n                        M #  >>  2  PMRP"); 
      printf("\n                        M #  >   4  PARTUR     [u scaling parameter]");
      printf("\n                        M #  >  10  PML - DCT");
      printf("\n                        M #  >  11  PML - UDWT 16x16");    
      printf("\n                        M #  >  14  PML - DCTst [-SoftThresh]");
      printf("\n                        M #  >  15  PML - DCTrsGM              [Qprior > GM  ::   g > DCT]");
      printf("\n                        M #  >  16  PML - DCTrsGM2             [Qprior Gamma > GM  :: g > DCT]");
      printf("\n                        M #  >  17  PML - DCT et GM alternance [g > GM :: Qprior > DCT]");
      printf("\n          >>> Tests reported in TBME article-----");
      printf("\n                        M #  >>  0  PGM");
      printf("\n                        M #  >>  1  PGGMRF     [Qprior]");
      printf("\n                        M #  >>  3  PMP        [Qprior=eta]");
      printf("\n                        M #  >> 12  PML - UDWT Whole Img");
      printf("\n                        M #  >> 13  PML - DCTrs [-RandomShifts]");  
      printf("\n                        M #  >> 18  PML - DCTrs with temp. decreasing");
      printf("\n                                      [Gamma.(Qprior)^NbIter] -> [min=Gamma/Value hyperpar]");
      printf("\n");
      printf("\n ---[Parameters]----------");
      printf("\n -n ? > Nb. Slices");
      printf("\n -t ? > Nb Iterations  (default =%d)",NBITERS);
      printf("\n -d ? > D with variance (default d=%d)",DVARIANCE);
      printf("\n -g ? > Gamma prior (default =%.1f)",GAMMA);
      printf("\n -b ? > Value hyperpar    [%.2f]",VALUE_HYPERPAR);
      printf("\n -r ? > Q prior (default =%.1f)",QPRIOR);
      printf("\n ---");
      printf("\n -U ? Cut-off:  [%d]",CUTOFF);
      printf("\n -W ? Cut-off2: [%d]",CUTOFF2);
      printf("\n");
      printf("\n ---[Bruits]-------------");
      printf("\n -B ? > NBGREYLEVELS (default:%d)",NbGreyLevels);
      printf("\n -C ? > WIDTH_BLUR (default:%d)",WidthBlur);
      printf("\n -l ? > With Poisson Noise [default (non) =%.1f)",VARNOISE);
      printf("\n -o ? > Scale Reduction [Nb. Counts] (default =%.1f)",SCALEREDUC);
      printf("\n");
      printf("\n ---[REAL SPECT INPUT FILE]--------");
      printf("\n -T ? Number of the Slice (%d)",SLC_SPECT);
      printf("\n -F ? Attenuation Factor: [%.2f]",FACTATTEN); 
      printf("\n -S <file1> <file2>  RealSPECTSinogramm (128x128x90)");
      printf("\n -X <file1> <file2>  RealSPECTSinogramm (224x224x45)");
      printf("\n");
      printf("\n ---[Flags]-------------");
      printf("\n -z Flag Zoom (%d)",ZOOM);
      printf("\n -q Flag Quit (Sans)"); 
      printf("\n -a Flag ISNR (Sans)"); 
      printf("\n -p Flag Test (Sans)"); 
      printf("\n");
      printf("\n ---[Sauvegarde]-------");
      printf("\n -u ?   Commentaire   [%s]",COMMENT);
      printf("\n ----------------------");

      printf("\n\n");
      exit(-1); }

      //Chargement Fichier
      strcpy(Name_img,argv[1]);

   //Boucle
   for(i=2;i<argc;i++)
     {   
          switch(argv[i][1])
          {
           case 'm': Method=atoi(argv[++i]);                 break;
	   case 'M': SubMethod=atoi(argv[++i]);              break;

           case 'B': NbGreyLevels=atoi(argv[++i]);           break;
           case 'C': WidthBlur=atoi(argv[++i]);              break;    

	   case 'i': init=atoi(argv[++i]);                   break;             
           case 'n': nbslices=atoi(argv[++i]);               break;    
           case 't': NbIters=atoi(argv[++i]);                break; 
           case 'd': Dvariance=atoi(argv[++i]);              break;
           case 'g': Gamma=atof(argv[++i]);                  break;
           case 'r': Qprior=atof(argv[++i]);                 break;          
           case 'l': varnoise=atof(argv[++i]);               break;  
           case 'o': scalereduc=atof(argv[++i]);             break;  
           case 'b': hyper=atof(argv[++i]);                  break;

           case 'T': slice_number=atoi(argv[++i]);           break;
           case 'F': factatten=atof(argv[++i]);              break;
           case 'S': flag_spect=1; 
                     strcpy(Name_spect,argv[++i]);           
                     strcpy(Name_spect2,argv[++i]);          break;
           case 'X': flag_spect=3; 
                     strcpy(Name_spect,argv[++i]);          
                     strcpy(Name_spect2,argv[++i]);          break;
           case 'U': cutoff=atoi(argv[++i]);                 break;
           case 'W': cutoff2=atoi(argv[++i]);                break; 
	   case 'u': strcpy(Comment,argv[++i]);              break;

           case 's': flag_save=1;                            break; 
           case 'q': flag_quit=1;                            break;
           case 'a': flag_tab=1;                             break;
           case 'p': flag_test=1;                            break;
           case 'z': zoom=atoi(argv[++i]);                   break;
           case '?':                                         break;                
          }    
     } 
 }

//----------------------------------------------------------
//----------------------------------------------------------
// PROGRAMME PRINCIPAL -------------------------------------
//----------------------------------------------------------
//----------------------------------------------------------
int main(int argc,char** argv)
{
 int flag;
 int length,width;

 //Pour Xwindow
 //------------
 XEvent ev;
 Window win_ppicture;
 Window win_ppicture_TransfRadon;
 Window win_ppicture_TransfRadonProj; 
 Window win_ppicture_TransfRadonEsti;
 Window win_ppicture_initreconst;
 Window win_ppicture_reconst;

 XImage *x_ppicture; 
 XImage *x_ppicture_TransfRadon;
 XImage *x_ppicture_TransfRadonProj;
 XImage *x_ppicture_TransfRadonEsti;
 XImage *x_ppicture_initreconst; 
 XImage *x_ppicture_reconst;
 
 char nomfen_ppicture[NCHAR];
 char nomfen_ppicture_TransfRadon[NCHAR];
 char nomfen_ppicture_TransfRadonProj[NCHAR];
 char nomfen_ppicture_TransfRadonEsti[NCHAR];
 char nomfen_ppicture_initreconst[NCHAR]; 
 char nomfen_ppicture_reconst[NCHAR];

 //Initialisation
 //--------------
 Qprior=QPRIOR;
 Gamma=GAMMA;
 Method=METHOD;
 SubMethod=SUBMETHOD;
 NbIters=NBITERS;
 Dvariance=DVARIANCE;
 init=INIT;
 nbslices=NBSLICES;

 cutoff=CUTOFF;
 cutoff2=CUTOFF2;

 flag_save=SAVE;
 flag_quit=QUIT;
 flag=0;
 zoom=ZOOM;
 flag_tab=TAB;
 flag_test=TEST;
 flag_spect=SPECT;

 slice_number=SLC_SPECT;
 rcf_factor=RCF_FACTOR; 
 factatten=FACTATTEN;
 hyper=VALUE_HYPERPAR;

 varnoise=VARNOISE;
 scalereduc=SCALEREDUC;
 NbGreyLevels=NB_GREYLEVELS;
 WidthBlur=WIDTH_BLUR;

 //Lis Arguments
 //-------------
 read_arguments(argc,argv);

 //Affiche options
 //---------------
 printf("\n");
 printf("\n\n Info: ");
 printf("\n --------");
 printf("\n Ouverture de l'image : <%s>",Name_img);
 printf("\n --------");
 printf("\n Init :   [%d]",init);
 printf("\n --------");
 printf("\n Methode :   [%d]",Method);
 printf("\n SubMethod : [%d]",SubMethod);
 printf("\n --------");
 printf("\n NbGreyLevels : [%d]",NbGreyLevels); 
 printf("\n WidthBlur    : [%d]",WidthBlur); 
 printf("\n --------");
 printf("\n Nb. Slices :  [%d]",nbslices);
 printf("\n NbIters   :   [%d]",NbIters); 
 printf("\n Dvariance :    [%d]",Dvariance); 
 printf("\n Gamma :        [%.2f]",Gamma);
 printf("\n Q-Prior :      [%.2f]",Qprior);
 printf("\n Value Hyper:   [%.2f]",hyper);
 printf("\n --------"); 
 printf("\n Var Noise    [%.2f]",varnoise);
 printf("\n Scale Reduction   [%.2f]",scalereduc);
 printf("\n --------");
 printf("\n SliceNumber:          [%d]",slice_number); 
 printf("\n RCF Factor :          [%.2f]",rcf_factor);
 printf("\n Facteur Attenuation : [%.2f]",factatten);
 printf("\n --------");
 printf("\n Commentaire:         [%s]",Comment);
 printf("\n ---------");
 printf("\n Zoom: [%d]",zoom); 

 if (flag_test)   printf("\n -> Avec Test");
 if (flag_tab)    printf("\n -> Avec Tableau ISNR");
 if (flag_save)   printf("\n -> Avec Sauvegarde");
 if (flag_quit)   printf("\n -> Avec Quit");

//-------------------------------
//---- Chargement des Images -------------------------------
//-------------------------------

  //Recuperation Images & Info
  //--------------------------
  GetLengthWidth(Name_img,&length,&width);
  int** datai=imatrix_allocate_2d(length,width);
  load_image_pgm(Name_img,datai,length,width);
 
  //Information 
  printf("\n Taille de l'image: [%d - %d]",length,width);
  fflush(stdout); 

  //-----------------------------------
  //---- Algorithme Tomograph ---------
  //-----------------------------------
  Tomograph tomo(length,width,nbslices);
  tomo.SimulateTomograph(datai);
 
  //-------------------------------------
  //--- Reconstruction ICD -------------- 
  //-------------------------------------
  ICD icd(length,width,nbslices,init);
  
  //Options
  icd.Options(datai);
  icd.Options2(Dvariance,NbIters,Method,Gamma,Qprior,flag_save,Name_img,flag_tab,varnoise,hyper);
  icd.Options3(scalereduc,flag_test,Comment);
  icd.Options4(NbGreyLevels,WidthBlur);
  icd.OptionFilter(cutoff,cutoff2);
  icd.OptionOthers(SubMethod,factatten,flag_spect);
  
  //Execution
  if (flag_spect==1) icd.OptionRealSPECT(Name_spect,Name_spect2,slice_number,rcf_factor); 
  if (flag_spect==3) icd.OptionRealSPECT2(Name_spect,Name_spect2,slice_number,rcf_factor);

  if (!flag_spect)   icd.Reconstruction(&tomo);
  if (flag_spect==1) icd.ReconstructionRealSPECT(&tomo);  
  if (flag_spect==3) icd.ReconstructionRealSPECT(&tomo);  
      

//---------------------------------------------------------------
//---------------------------------------------------------------
//---------------- visu sous XWINDOW ----------------------------
//---------------------------------------------------------------
 if (!flag_quit)
 {
 //ouverture session graphique
 if (open_display()<0) printf(" Impossible d'ouvrir une session graphique");

 //commentaire dans les photos affichees
 sprintf(nomfen_ppicture,"Image : %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadon,"TransfRadon 1SB: %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadonProj,"TransfRadonFilt: %s",Name_img);
 sprintf(nomfen_ppicture_TransfRadonEsti,"TransfRadonEsti : %s",Name_img);
 sprintf(nomfen_ppicture_initreconst,"Image InitReconst: %s",Name_img);
 sprintf(nomfen_ppicture_reconst,"Image Reconst: %s",Name_img);
 
 //creation des fenetres
 win_ppicture=fabrique_window(nomfen_ppicture,10,10,width,length,zoom);
 win_ppicture_TransfRadon=fabrique_window(nomfen_ppicture_TransfRadon,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_TransfRadonProj=fabrique_window(nomfen_ppicture_TransfRadonProj,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_TransfRadonEsti=fabrique_window(nomfen_ppicture_TransfRadonEsti,10,10,tomo.width_radon,tomo.length_radon,zoom);
 win_ppicture_initreconst=fabrique_window(nomfen_ppicture_initreconst,10,10,icd.width,icd.length,zoom);
 win_ppicture_reconst=fabrique_window(nomfen_ppicture_reconst,10,10,icd.width,icd.length,zoom);
 
 //creation des Ximages
 x_ppicture=cree_Ximage4(datai,zoom,length,width);
 x_ppicture_TransfRadon=cree_Ximage2(icd.mat_radon_visu,zoom,icd.length_radon,icd.width_radon);
 x_ppicture_TransfRadonProj=cree_Ximage2(icd.mat_radonProj_visu,zoom,icd.length_radon,icd.width_radon);
 x_ppicture_TransfRadonEsti=cree_Ximage2(icd.mat_radonEsti_visu,zoom,icd.length_radon,icd.width_radon);
 x_ppicture_initreconst=cree_Ximage2(icd.mat_initreconst,zoom,icd.length,icd.width);
 x_ppicture_reconst=cree_Ximage2(icd.mat_reconst,zoom,icd.length,icd.width);
 
 printf("\n\n Pour quitter,appuyer sur la barre d'espace");

 //boucle d'evenements
  for(;;)
     {
      XNextEvent(display,&ev);
       switch(ev.type)
        {
         case Expose:
         XPutImage(display,win_ppicture,gc,x_ppicture,0,0,0,0,
                   x_ppicture->width,x_ppicture->height);
         XPutImage(display,win_ppicture_TransfRadon,gc,x_ppicture_TransfRadon,0,0,0,0,
                   x_ppicture_TransfRadon->width,x_ppicture_TransfRadon->height);
         XPutImage(display,win_ppicture_TransfRadonProj,gc,x_ppicture_TransfRadonProj,0,0,0,0,
                   x_ppicture_TransfRadonProj->width,x_ppicture_TransfRadonProj->height);
         XPutImage(display,win_ppicture_TransfRadonEsti,gc,x_ppicture_TransfRadonEsti,0,0,0,0,
                   x_ppicture_TransfRadonEsti->width,x_ppicture_TransfRadonEsti->height);
         XPutImage(display,win_ppicture_initreconst,gc,x_ppicture_initreconst,0,0,0,0,
                   x_ppicture_initreconst->width,x_ppicture_initreconst->height);
         XPutImage(display,win_ppicture_reconst,gc,x_ppicture_reconst,0,0,0,0,
                   x_ppicture_reconst->width,x_ppicture_reconst->height);
         break;

         case KeyPress:
         XDestroyImage(x_ppicture);
         XDestroyImage(x_ppicture_TransfRadon); 
         XDestroyImage(x_ppicture_TransfRadonProj);
         XDestroyImage(x_ppicture_TransfRadonEsti);
         XDestroyImage(x_ppicture_initreconst);
         XDestroyImage(x_ppicture_reconst);

         XFreeGC(display,gc);
         XCloseDisplay(display);
         flag=1;
         break;
         }
   if (flag==1) break;
   }
 } 
       
//--------------- fin de la session graphique --------------     
//----------------------------------------------------------

   //Liberation Memoire
   //------------------
   if (datai) free_imatrix_2d(datai);
 
   //retour sans probleme
   //--------------------
   printf("\n\n\n C'est fini... \n\n\n\n\n");
   exit(-1);
   return 0;
 }
 


