//---------------------------------------------------
// module  : ICD.h                         
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                 
//---------------------------------------------------
// 

#ifndef ICD_H
#define ICD_H

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include <stdio.h>

#include "Function.h"
#include "Tomograph.h"
#include "FilteredRetroProjection.h"
#include "DCT.h"
#include "Haar.h"


//------------------------------------------------
// DEFINITIONS -----------------------------------
//------------------------------------------------
#define NBCHARS 200
#define NCHAR 200
#define BIG_NUMBER 100000
#define MILLION 1000000
#define MILLE 1000

#define INIT_FBP 0
#define INIT_CST 2

#define CLIQUE_HVER   0.146447
#define CLIQUE_DIAG   0.103553

#define NBGREYLEVELS        256
#define NBGREYLEVELSMAX     256
#define ITER_MAX             50
#define ITER_MINI            10
#define BBIG_NUMBER   100000000
#define DIXMILLE          10000
#define NB_HYPERPAR           4

#define LENGTH_SPECT 128
#define WIDTH_SPECT  128
#define NBSLC_SPECT   90
#define SLC_SPECT     47  
#define RCF_FACTOR    0.8

#define CUTOFF        1000
#define CUTOFF2       30

//------------------------------------------------
// CLASSE ICD  -----------------------------------
//------------------------------------------------
class ICD
{
 //-- Donnees --
 //-------------
 public:
   //MatricesVecteur
   float**  mat_img;
   float**  mat_initreconst;
   float**  mat_initreconstsave; 
   float**  mat_reconst;
   float**  mat_reconstBestMAP;
   float**  mat_reconstBestMAPTest;
   float**  mat_reconst_prec;
   float**  mat_tmp1;
   float**  mat_tmp2; 
   float**  mat_tmp3; 
   float**  mat_tmp4;  
   float**  mat_radon;
   float**  mat_radonWithoutNoise;
   float**  mat_radonProj;
   float**  mat_radonProjWithoutNoise;
   float**  mat_radonNoise;
   float**  mat_radon_visu;
   float**  mat_radonProj_visu;
   float**  mat_radonEsti;
   float**  mat_radonEsti_visu;

   float**   Project;
   float***  ShortProject;
   float***  ShortProject2;

   float*   DiagVariance;
   float*   VectProj;
   float*   VectProjError;

   float*   VectImgF;      // Image reconstruite courant mis en Vect
   float*   Vecty;         // Matrice Radon calc.avec Project (vecty = P.VectImg)
   float*   VectNoise;
   float*   VectyWithoutNoise;  

   float*   VectImg;       // Image mat_img mis en VectImg
   float*   VectHist;      // Vecteur Histogramme    
   
   //Tailles
   int width,length;
   int size;
   int width_radon,length_radon;
   int nbslices;
   int width_proj,length_proj;
   int width_shortproj;
   int length_shortproj;

   //Donnees
   float MoyImg;

   //Phantom & Algo
   float  ScaleReduc;
   int    NbGreyLevels;
   int    WidthBlur;
   float  SigmaBlur;
   float  ScalePhantom;
   float  varnoise;

   //Option-Parametre
   int    Method;
   int    SubMethod;
   int    init;
   int    NbIters;
   float  Gamma;
   float  Qprior;
   int    Dvariance;

   //Pour Tests
   float   Gamma_init;
   float   Gamma_final;
   float** TestResults;
   
   //NRJ-Mesure
   float NrjMAP;
   float NrjLikelihood;
   float NrjPrior;
   float SNR;
   float MMSE;
   float Best_SNR;
   int   Best_iter;
   float BestMAP_SNR;
   float BestMAP_SNRTest;
   float BestMAP_GammaTest;
   float BestMAP_MMSE;
   int   BestMAP_iter;
   float Stop_SNR;
   float* TabSnr;
   float  convmap;
   float  convimg;

   //Algorithme ICDs
   float**  Median;
   float**  Bx;
   float**  By;
   float**  Bd1;
   float**  Bd2;

   //Filtrage
   int   CuttOff; 
   int   CuttOff2;
   float FactAtten;

   //Flags
   int  flag_save;
   int  flagR_silent;
   int  flag_tab;
   int  flag_tests;
   int  flag_ShortProject;
   int  flag_RealSPECT;
   int  flag_Spect;
   int  flag_Sym;

   //Buf. Utile
   char File_Test[NCHAR];
   char File_Result[NCHAR]; 
   char Par_Recons[NCHAR];
   char Name_Sauve[NCHAR];
   char Name2_Sauve[NCHAR];
   char Name_Img[NCHAR];
   char Comment[NCHAR];

   //Parametre
   //---------
   float    value_hyperpar;

   //-------------------
   //Pour SinoRealSPECT
   //-------------------
   float*** SinoRealSPECT;
   float*** SinoRealSPECT2;
   float*** ImgSinoRealSPECT;
   float*** ImgSinoRealSPECT2;
   float*** CoupeProjSPECT;
   float*** CoupeProjSPECTImag;
   float*** ImgCoupeProjSPECT;
   float**  SinoRealSPECT2Raw;
   int      SliceNumber;
   float    rcf_factor;

   //------
   //Objets
   //------
   DCT*               dct;
   Haar*             udwt;
 
  //--------------
  //-- Methodes --
  //--------------
  public:
    ICD(int,int,int,int);
    ~ICD();

   //Data
   //----------
   //>Synth
   void Reconstruction(Tomograph*);

   //>Reel Soucy
   void ReconstructionRealSPECT(Tomograph*);
   
   //Fonction EM
   //-----------
   void PE_EM_Init();
   void PE_EM();
   
   //-------------------------------------------------
   //Fichier f_ICD1.cc
   //-------------------------------------------------
   //Options
   void Options(int**);
   void Options2(int,int,int,float,float,int,char*,int,float,float);
   void Options3(float,int,char*);
   void Options4(int,int);
   void OptionFilter(int,int);
   void OptionOthers(int,float,int);
   void OptionRealSPECT(char*,char*,int,float); 
   void OptionRealSPECT2(char*,char*,int,float); 
   void OptionRealSINOG(char*,char*,float); 
     
   //->Init
   //------
   void ICD_Init();
   
   //->Fonctions de Base
   //-------------------
   void  ComputeProjectMatrix();
   void  ComputeRadonMatrix();
   void  ComputeNewShortRadonMatrix(float**,float**); 
   void  ComputePoissonNoise(float**,int,int);
   void  ComputeRSBNoise(float**,float**,int,int);
   void  AddPoissonNoiseToVectyFromTomo(); 
   void  ComputeProjectionError();
   void  ComputeVarianceProjection();
   float ComputeLikelihood();

   //->Optimisation-Projection
   //--------------------------
   void  ComputeNewShortProjectMatrix();
   void  ComputeNewShortProjectionError();
   
   //PRIOR
   //=====
   //--------------------
   //>Prior GaussMarkov
   //--------------------
   float  ICD0_ComputeGMderive(int,int);
   float  ICD0_ComputeMAP2();
   
   //--------------
   //>Prior GGMRF
   //--------------
   float  ICD1_ComputeTheta1(int);
   float  ICD1_ComputeTheta2(int);
   void   ICD1_UpdateProjection(int,float,float);
   float  ICD1_ComputeGGMRFderive(int,int,float);
   float  ICD1_ComputeMAP();

   //------------
   //>Prior MRP
   //------------
   float  ICD2_ComputeMRPderive(int,int,float,float**);
   float  ICD2_ComputeMAP(float**);

   //-----------
   //>Prior <MP>
   //-----------
   void   ICD3_MedianFilter(float**,int);
   float  ICD3_ComputeMPderive(int,int,float,float**);
   float  ICD3_ComputeMAP(float**);

   //---------------
   //>Prior <ARTUR>
   //--------------- 
   float  ICD4_ComputeMAP();
   void   ICD4_computeBxByBd(float**,int);
   float  ICD4_ComputeARTURderive(int,int,float);
  
   
   //Utile
   //-----
   void   ComputeBiasVariance(float**,float**);
   float  ComputeSNR(float**,float**);
   float  ComputeMMSE(float**,float**);
   float  ComputeError(float**,float**,int,int);
   float  ComputeError2(float**,float**,int,int);
   void   ConvertMatVct(float**,float*,int,int); 
   void   MedianFilter2(float**,int);
   float  PhidGM(float);
   float  PsiGM(float);
   void   Save_picture_pgm(char*,char*,char*,float**,int,int);
   void   RecalSave_picture_pgm(char*,char*,char*,float**,int,int);
   void   RegionBasedNConstraintsDeriveForGradient(float**,float**);
   void   FillDataReconstruction();
   void   FillDataResReconstruction();
   void   FillResReconstruction();
   void   FillResReconstruction2();
   void   AfficheTabSnr(float*);
   float  ComputeL2Error(float,float);
   void   InfoPhantom(float**,int); 
   void   LSCorrectionInitImg(float);
   void   LSCorrectionInitSPECTImg(); 
   void   AddBlurOnMatrix(float**,int,int,float);
   void   AddBlurOnVector(float*,int,int,float);
   float  Funcgauss(short,float);
   void   ButterworthFilter(float**,int,int,int,float);
   void   ButterworthFilterNS(float**,int,int,int,float);
   void   ButterworthFilter3D(float***,int,int,int,int,int);
   void   Center3D(float***,int,int,int);
};

#endif 
