//------------------------------------------------------
// module  : f_DCT.cc
// auteur  : Mignotte Max
// date    : 
// version : 1.0
// langage : C++
// labo    : DIRO
// note    : 
//------------------------------------------------------
// doit etre utilis (i.e ajout a la compile) par tous les prog. ecrit
// en C++ utilisant la classe DCT ou une de ses classes drive.
// le fichier header correspondant s'appelle DCT.h

//------------------------------------------------
// FICHIERS INCLUS -------------------------------
//------------------------------------------------
#include "DCT.h"

//------------------------------------------------
// CONSTANTES ------------------------------------
//------------------------------------------------
# define DEBUG 0

//------------------------------------------------
// CONS/DES DE LA CLASSE DCT ---------------------
//------------------------------------------------
//----------------------------------------------------------
// Constructeur de la classe DCT
//----------------------------------------------------------
DCT::DCT(int lgth,int wdth,int sz)
 {
  int i,j,k;

   //Enregistrement
   length=lgth;
   width=wdth;
   size=sz;

   //Allocation memoire matrice
   SquWin=fmatrix_allocate_2d(size,size);
   mat3d=fmatrix_allocate_3d(size*size,length,width);

    //Initialisation
    for(k=0;k<(size*size);k++)
    for(i=0;i<length;i++) for(j=0;j<width;j++)   
       mat3d[k][i][j]=-1.0;

   //Flag
   FlagSilent=0;
 }

//----------------------------------------------------------
// Constructeur de la classe DCT
//----------------------------------------------------------
DCT::DCT(int lgth,int wdth)
 {
  int i,j,k;

   //Enregistrement
   length=lgth;
   width=wdth;
   size=1;

   //Allocation memoire matrice
   SquWin=fmatrix_allocate_2d(length,width);
   mat3d=fmatrix_allocate_3d(size*size,length,width);

   //Initialisation
   for(k=0;k<(size*size);k++)
   for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;  

   //Flag
   FlagSilent=0;
 }

//----------------------------------------------------------
// Destructeur de la classe DCT
//----------------------------------------------------------
DCT::~DCT()
 { ; }


//-----------------------------------------------------------------
// METHODES DE LA CLASSE DCT --------------------------------------
//-----------------------------------------------------------------
//----------------------------------------------------------
// Fast FilteringDCT 8x8        
//----------------------------------------------------------
void DCT::FilteringDCT8x8_(float** imgin,float sigma,int ovl)
{
 int i,j;
 int k,l;
 int x,y;
 int pos;
 float temp;
 float nb;
 float MeanImg;
 int posr,posc;

 //Initialisation
 //--------------
 //>Enregistrement
 overlap=ovl;
 size=8;       

 //>Alloc
 mat3d=fmatrix_allocate_3d(size*size,length,width);

 //>Init
 for(k=0;k<(size*size);k++)
 for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;
 
 //Info
 //----
 if (!FlagSilent)
    {
     printf("\n\n");
     printf("\n  Hard Thresh. DCT ");
     printf("\n  ----------------- ");
     printf("\n   >> Thresh=    [%.2f]",sigma);
     printf("\n   >> SizeWind=  [%d]",size);
     printf("\n   >> Overlap=   [%d]",overlap);
     printf("\n"); 
    }

 //Init
 //----
 MeanImg=MeanSqWindow(imgin,length);
 if (!FlagSilent) printf("\n   > MoyImg Bruite [%.6f]",MeanImg);

 //Boucle
 //------
 for(i=0;i<length;i+=overlap) for(j=0;j<width;j+=overlap) 
   {
     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
	 posr=i-(size/2)+k;
         posc=j-(size/2)+l;

         if (posr<0)           posr+=length;
         if (posr>(length-1))  posr-=length; 
 
         if (posc<0)           posc+=width;
         if (posc>(width-1))   posc-=width;

         SquWin[k][l]=imgin[posr][posc];
        }

     ddct8x8s(-1,SquWin);
     HardThreshold(sigma,SquWin,size);
     ddct8x8s(1,SquWin); 
    
     x=(i%size);
     y=(j%size);
     pos=((x*size)+y);

     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
          posr=i-(size/2)+k;
          posc=j-(size/2)+l;

          if (posr<0)           posr+=length;
          if (posr>(length-1))  posr-=length; 
 
          if (posc<0)           posc+=width;
          if (posc>(width-1))   posc-=width;

          if  (mat3d[pos][posr][posc]!=-1) printf("!");

          mat3d[pos][posr][posc]=SquWin[k][l]; 
         }
   }

 //Moyennage
 //---------
 for(i=0;i<length;i++) for(j=0;j<width;j++)
    {
     temp=0.0; nb=0.0;
     for(k=0;k<(size*size);k++)
       if (mat3d[k][i][j]>0) { nb++; temp+=mat3d[k][i][j]; }
        
     if (nb) { temp/=nb;  imgin[i][j]=temp; }
    }

 //Info
 //----
 if (!FlagSilent)
    {
     MeanImg=MeanSqWindow(imgin,length);
     printf("\n   > MoyImg Restaure [%.6f]",MeanImg);
    }

 //Clear
 //-----
  for(k=0;k<(size*size);k++)
  for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;

 //Liberation Memoire
 if (mat3d) free_fmatrix_3d(mat3d,size*size);
}

//----------------------------------------------------------
// Fast FilteringDCT 8x8 Random Shift      
//----------------------------------------------------------
void DCT::FilteringDCT8x8rs_(float** imgin,float sigma,int ovl)
{
 int i,j;
 int k,l;
 int x,y;
 int pos;
 float temp;
 float nb;
 float MeanImg;
 int posr,posc;
 int nbalc,nbalr;

 //Initialisation
 //--------------
 //>Enregistrement
 overlap=ovl;
 size=8;       

 //>Alloc
 mat3d=fmatrix_allocate_3d(size*size,length,width);

 //>Init
 for(k=0;k<(size*size);k++)
 for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;
 
 //Info
 //----
 if (!FlagSilent)
    {
     printf("\n\n");
     printf("\n  Hard Thresh. DCT ");
     printf("\n  ----------------- ");
     printf("\n   >> Thresh=    [%.2f]",sigma);
     printf("\n   >> SizeWind=  [%d]",size);
     printf("\n   >> Overlap=   [%d]",overlap);
     printf("\n"); 
    }

 //Init
 //----
 MeanImg=MeanSqWindow(imgin,length);
 if (!FlagSilent) printf("\n   > MoyImg Bruite [%.6f]",MeanImg);

 //Random Shift
 overlap=3;
 nbalr=(int)random_flo(7.5); 
 nbalc=(int)random_flo(7.5); 
 //printf(" [>%d<>%d<>sig:%.2f]",nbalr,nbalc,sigma);

 //Boucle
 //------
 for(i=0;i<length;i+=overlap) for(j=0;j<width;j+=overlap) 
   {
     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
	 posr=i-(size/2)+k+nbalr;
         posc=j-(size/2)+l+nbalc;

         if (posr<0)           posr+=length;
         if (posr>(length-1))  posr-=length; 
 
         if (posc<0)           posc+=width;
         if (posc>(width-1))   posc-=width;

         SquWin[k][l]=imgin[posr][posc];
        }

     ddct8x8s(-1,SquWin);
     HardThreshold(sigma,SquWin,size);
     ddct8x8s(1,SquWin); 
    
     x=(i%size);
     y=(j%size);
     pos=((x*size)+y);

     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
          posr=i-(size/2)+k+nbalr;
          posc=j-(size/2)+l+nbalc;

          if (posr<0)           posr+=length;
          if (posr>(length-1))  posr-=length; 
 
          if (posc<0)           posc+=width;
          if (posc>(width-1))   posc-=width;

          if  (mat3d[pos][posr][posc]!=-1) printf("!");

          mat3d[pos][posr][posc]=SquWin[k][l]; 
         }
   }

 //Moyennage
 //---------
 for(i=0;i<length;i++) for(j=0;j<width;j++)
    {
     temp=0.0; nb=0.0;
     for(k=0;k<(size*size);k++)
       if (mat3d[k][i][j]>0) { nb++; temp+=mat3d[k][i][j]; }
        
     if (nb) { temp/=nb;  imgin[i][j]=temp; }
    }

 //Info
 //----
 if (!FlagSilent)
    {
     MeanImg=MeanSqWindow(imgin,length);
     printf("\n   > MoyImg Restaure [%.6f]",MeanImg);
    }

 //Clear
 //-----
  for(k=0;k<(size*size);k++)
  for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;

 //Liberation Memoire
 if (mat3d) free_fmatrix_3d(mat3d,size*size);
}

//----------------------------------------------------------
// Fast FilteringDCT 8x8        
//----------------------------------------------------------
void DCT::FilteringDCT8x8st_(float** imgin,float sigma,int ovl)
{
 int i,j;
 int k,l;
 int x,y;
 int pos;
 float temp;
 float nb;
 float MeanImg;
 int posr,posc;

 //Initialisation
 //--------------
 //>Enregistrement
 overlap=ovl;
 size=8;       

 //>Alloc
 mat3d=fmatrix_allocate_3d(size*size,length,width);

 //>Init
 for(k=0;k<(size*size);k++)
 for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;
 
 //Info
 //----
 if (!FlagSilent)
    {
     printf("\n\n");
     printf("\n  Hard Thresh. DCT ");
     printf("\n  ----------------- ");
     printf("\n   >> Thresh=    [%.2f]",sigma);
     printf("\n   >> SizeWind=  [%d]",size);
     printf("\n   >> Overlap=   [%d]",overlap);
     printf("\n"); 
    }

 //Init
 //----
 MeanImg=MeanSqWindow(imgin,length);
 if (!FlagSilent) printf("\n   > MoyImg Bruite [%.6f]",MeanImg);

 //Boucle
 //------
 for(i=0;i<length;i+=overlap) for(j=0;j<width;j+=overlap) 
   {
     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
	 posr=i-(size/2)+k;
         posc=j-(size/2)+l;

         if (posr<0)           posr+=length;
         if (posr>(length-1))  posr-=length; 
 
         if (posc<0)           posc+=width;
         if (posc>(width-1))   posc-=width;

         SquWin[k][l]=imgin[posr][posc];
        }

     ddct8x8s(-1,SquWin);
     SoftThreshold2(sigma,SquWin,8); 
     ddct8x8s(1,SquWin); 
    
     x=(i%size);
     y=(j%size);
     pos=((x*size)+y);

     for(k=0;k<size;k++) for(l=0;l<size;l++) 
        { 
          posr=i-(size/2)+k;
          posc=j-(size/2)+l;

          if (posr<0)           posr+=length;
          if (posr>(length-1))  posr-=length; 
 
          if (posc<0)           posc+=width;
          if (posc>(width-1))   posc-=width;

          if  (mat3d[pos][posr][posc]!=-1) printf("!");

          mat3d[pos][posr][posc]=SquWin[k][l]; 
         }
   }

 //Moyennage
 //---------
 for(i=0;i<length;i++) for(j=0;j<width;j++)
    {
     temp=0.0; nb=0.0;
     for(k=0;k<(size*size);k++)
       if (mat3d[k][i][j]>0) { nb++; temp+=mat3d[k][i][j]; }
        
     if (nb) { temp/=nb;  imgin[i][j]=temp; }
    }

 //Info
 //----
 if (!FlagSilent)
    {
     MeanImg=MeanSqWindow(imgin,length);
     printf("\n   > MoyImg Restaure [%.6f]",MeanImg);
    }

 //Clear
 //-----
  for(k=0;k<(size*size);k++)
  for(i=0;i<length;i++) for(j=0;j<width;j++)   
      mat3d[k][i][j]=-1.0;

 //Liberation Memoire
 if (mat3d) free_fmatrix_3d(mat3d,size*size);
}

//--------------------------//
// Threshold DCT -----------//
//--------------------------//
//----------------------------------------------------------
//  DCT thresholding         
//----------------------------------------------------------
void DCT::HardThreshold(float sigma,float** coef,int N)
{
 int i,j; 

 for(i=0;i<N;i++) for(j=0;j<N;j++) if (fabs(coef[i][j])<sigma) coef[i][j]=0.0;
}

//----------------------------------------------------------
//  DCT thresholding         
//----------------------------------------------------------
void DCT::SoftThreshold2(float sigma,float** coef,int N)
{
 int i,j; 
 int signe;

 for(i=0;i<N;i++) for(j=0;j<N;j++) 
    { 
     signe=1;
     if (coef[i][j]<0)  signe=-1;
     if (coef[i][j]>=0) signe=1;

     coef[i][j]=signe*MAXIMUM(0,(fabs(coef[i][j])-sigma));
    }
}

//--------------------------//
// FAST DCT ----------------//
//--------------------------//
#define C8_1R   0.49039264020161522456
#define C8_1I   0.09754516100806413392
#define C8_2R   0.46193976625564337806
#define C8_2I   0.19134171618254488586
#define C8_3R   0.41573480615127261854
#define C8_3I   0.27778511650980111237
#define C8_4R   0.35355339059327376220
#define W8_4R   0.70710678118654752440

void DCT::ddct8x8s(int isgn, float **a)
{
    int j;
    float x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    float xr, xi;
    
    if (isgn < 0) {
        for (j = 0; j <= 7; j++) {
            x0r = a[0][j] + a[7][j];
            x1r = a[0][j] - a[7][j];
            x0i = a[2][j] + a[5][j];
            x1i = a[2][j] - a[5][j];
            x2r = a[4][j] + a[3][j];
            x3r = a[4][j] - a[3][j];
            x2i = a[6][j] + a[1][j];
            x3i = a[6][j] - a[1][j];
            xr = x0r + x2r;
            xi = x0i + x2i;
            a[0][j] = C8_4R * (xr + xi);
            a[4][j] = C8_4R * (xr - xi);
            xr = x0r - x2r;
            xi = x0i - x2i;
            a[2][j] = C8_2R * xr - C8_2I * xi;
            a[6][j] = C8_2R * xi + C8_2I * xr;
            xr = W8_4R * (x1i - x3i);
            x1i = W8_4R * (x1i + x3i);
            x3i = x1i - x3r;
            x1i += x3r;
            x3r = x1r - xr;
            x1r += xr;
            a[1][j] = C8_1R * x1r - C8_1I * x1i;
            a[7][j] = C8_1R * x1i + C8_1I * x1r;
            a[3][j] = C8_3R * x3r - C8_3I * x3i;
            a[5][j] = C8_3R * x3i + C8_3I * x3r;
        }
        for (j = 0; j <= 7; j++) {
            x0r = a[j][0] + a[j][7];
            x1r = a[j][0] - a[j][7];
            x0i = a[j][2] + a[j][5];
            x1i = a[j][2] - a[j][5];
            x2r = a[j][4] + a[j][3];
            x3r = a[j][4] - a[j][3];
            x2i = a[j][6] + a[j][1];
            x3i = a[j][6] - a[j][1];
            xr = x0r + x2r;
            xi = x0i + x2i;
            a[j][0] = C8_4R * (xr + xi);
            a[j][4] = C8_4R * (xr - xi);
            xr = x0r - x2r;
            xi = x0i - x2i;
            a[j][2] = C8_2R * xr - C8_2I * xi;
            a[j][6] = C8_2R * xi + C8_2I * xr;
            xr = W8_4R * (x1i - x3i);
            x1i = W8_4R * (x1i + x3i);
            x3i = x1i - x3r;
            x1i += x3r;
            x3r = x1r - xr;
            x1r += xr;
            a[j][1] = C8_1R * x1r - C8_1I * x1i;
            a[j][7] = C8_1R * x1i + C8_1I * x1r;
            a[j][3] = C8_3R * x3r - C8_3I * x3i;
            a[j][5] = C8_3R * x3i + C8_3I * x3r;
        }
    } else {
        for (j = 0; j <= 7; j++) {
            x1r = C8_1R * a[1][j] + C8_1I * a[7][j];
            x1i = C8_1R * a[7][j] - C8_1I * a[1][j];
            x3r = C8_3R * a[3][j] + C8_3I * a[5][j];
            x3i = C8_3R * a[5][j] - C8_3I * a[3][j];
            xr = x1r - x3r;
            xi = x1i + x3i;
            x1r += x3r;
            x3i -= x1i;
            x1i = W8_4R * (xr + xi);
            x3r = W8_4R * (xr - xi);
            xr = C8_2R * a[2][j] + C8_2I * a[6][j];
            xi = C8_2R * a[6][j] - C8_2I * a[2][j];
            x0r = C8_4R * (a[0][j] + a[4][j]);
            x0i = C8_4R * (a[0][j] - a[4][j]);
            x2r = x0r - xr;
            x2i = x0i - xi;
            x0r += xr;
            x0i += xi;
            a[0][j] = x0r + x1r;
            a[7][j] = x0r - x1r;
            a[2][j] = x0i + x1i;
            a[5][j] = x0i - x1i;
            a[4][j] = x2r - x3i;
            a[3][j] = x2r + x3i;
            a[6][j] = x2i - x3r;
            a[1][j] = x2i + x3r;
        }
        for (j = 0; j <= 7; j++) {
            x1r = C8_1R * a[j][1] + C8_1I * a[j][7];
            x1i = C8_1R * a[j][7] - C8_1I * a[j][1];
            x3r = C8_3R * a[j][3] + C8_3I * a[j][5];
            x3i = C8_3R * a[j][5] - C8_3I * a[j][3];
            xr = x1r - x3r;
            xi = x1i + x3i;
            x1r += x3r;
            x3i -= x1i;
            x1i = W8_4R * (xr + xi);
            x3r = W8_4R * (xr - xi);
            xr = C8_2R * a[j][2] + C8_2I * a[j][6];
            xi = C8_2R * a[j][6] - C8_2I * a[j][2];
            x0r = C8_4R * (a[j][0] + a[j][4]);
            x0i = C8_4R * (a[j][0] - a[j][4]);
            x2r = x0r - xr;
            x2i = x0i - xi;
            x0r += xr;
            x0i += xi;
            a[j][0] = x0r + x1r;
            a[j][7] = x0r - x1r;
            a[j][2] = x0i + x1i;
            a[j][5] = x0i - x1i;
            a[j][4] = x2r - x3i;
            a[j][3] = x2r + x3i;
            a[j][6] = x2i - x3r;
            a[j][1] = x2i + x3r;
        }
    }
}

//---------------
// GESTION ------
//---------------
//----------------------------------------------------------
// MeanSqWindow             
//----------------------------------------------------------     
float DCT::MeanSqWindow(float** mat,int sz)
{
 int i,j;
 float mean=0.0;

 for(i=0;i<sz;i++) for(j=0;j<sz;j++)
   mean+=mat[i][j];

 mean/=(CARRE(sz));

 return mean;
}

