//---------------------------------------------------
// module  : f_Haar.cc                         
// auteur  : Max Mignotte                           
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//---------------------------------------------------
// FICHIERS INCLUS ----------------------------------
//---------------------------------------------------
#include "Haar.h"

//----------------------------------------------------
// CONSTRUCTEUR ET DESTRUCTEUR DE LA CLASSE Haar -----
//----------------------------------------------------
//----------------------------------------------------------
// constructeur de la classe Haar 
//----------------------------------------------------------
 Haar::Haar(int Length,int Width) 
 { 
  //--init. Longeur/Largeur--
  width=Width; length=Length;
  size=width;
  nlev_udwt=(int)(log((float)width)/log(2.0));
  printf("\n  Objet Haar");
  printf("\n  ----------");
  printf("\n     >Nb level UDWT=%d",nlev_udwt);


  //--Width=Length?--
  if (width!=length)
    { printf("\n Length et Width differents");
      exit(-1); }

  //--init matrice --
  mat=fmatrix_allocate_2d(length,width);
  mat_dec=fmatrix_allocate_2d(length,width);
  mat_rec=fmatrix_allocate_2d(length,width);
  mat_decvisu=fmatrix_allocate_2d(length,width);

  //--Init Vecteur--
  vect=fmatrix_allocate_1d(width);
  vect_dec=fmatrix_allocate_1d(width); 
  vect_rec=fmatrix_allocate_1d(width);
  vect_tmp=fmatrix_allocate_1d(width);

  //Init matrice UDWT 
  mat_udwt=fmatrix_allocate_2d(length*nlev_udwt,width*nlev_udwt);
  mat_udwt_rec=fmatrix_allocate_2d(length*nlev_udwt,width*nlev_udwt);
  mat_udwt_dec=fmatrix_allocate_2d(length*nlev_udwt,width*nlev_udwt);

  vect_udwt=fmatrix_allocate_1d(width*nlev_udwt);
  vect_udwt_dec=fmatrix_allocate_1d(width*nlev_udwt); 
  vect_udwt_rec=fmatrix_allocate_1d(width*nlev_udwt);
  vect_udwt_tmp=fmatrix_allocate_1d(width*nlev_udwt);
  vect_udwt_tmp2=fmatrix_allocate_1d(width*nlev_udwt);
  vect_udwt_tmp3=fmatrix_allocate_1d(width*nlev_udwt);
  vect_iudwt=fmatrix_allocate_1d(width*nlev_udwt);
  vect_iudwt_tmp=fmatrix_allocate_1d(width*nlev_udwt);
  vect_iudwt_tmp2=fmatrix_allocate_1d(width*nlev_udwt); 
  vect_iudwt_tmp3=fmatrix_allocate_1d(width*nlev_udwt); 
  tabvect_iudwt=fmatrix_allocate_2d(2,width*nlev_udwt); 

  //--Flag------
  FlagComment=0;
 
 }
//----------------------------------------------------------
// destructeur de la classe Haar
//----------------------------------------------------------
 Haar::~Haar()
 { 
   //Liberation memoire matrice--
   free_fmatrix_2d(mat); 
   free_fmatrix_2d(mat_dec);
   free_fmatrix_2d(mat_rec);
   free_fmatrix_2d(mat_decvisu);

   //Liberation memoire vecteur--
   free_fmatrix_1d(vect); 
   free_fmatrix_1d(vect_dec);
   free_fmatrix_1d(vect_rec);
   free_fmatrix_1d(vect_tmp);

   //UDWT
   free_fmatrix_2d(mat_udwt);
   free_fmatrix_2d(mat_udwt_rec);
   free_fmatrix_2d(mat_udwt_dec);
 
   free_fmatrix_1d(vect_udwt);
   free_fmatrix_1d(vect_udwt_dec);
   free_fmatrix_1d(vect_udwt_rec);
   free_fmatrix_1d(vect_udwt_tmp);
   free_fmatrix_1d(vect_udwt_tmp2); 
   free_fmatrix_1d(vect_udwt_tmp3); 
   free_fmatrix_1d(vect_iudwt);
   free_fmatrix_1d(vect_iudwt_tmp); 
   free_fmatrix_1d(vect_iudwt_tmp2); 
   free_fmatrix_1d(vect_iudwt_tmp3); 
   free_fmatrix_2d(tabvect_iudwt); 
 }


//------------------------------------------------
// METHODES DE LA CLASSE Haar --------------------
//------------------------------------------------
//-------------------//
//-- DECOMPOSITION --//
//-------------------//
//----------------------------------------------------------
// Decompose2D
//----------------------------------------------------------
void Haar::DecomposeHaar2D(int nlev)
 {
 int i,j,k,l;
 int sz;

  //Presentation
  if (FlagComment)
     {
      printf("\n\n\n");
      printf("\n --------------------");
      printf("\n Decomposition Image ");
      printf("\n --------------------");
      printf("\n");
     }

  //Initialisation
  NbLevels=nlev;
  sz=size;
  if (FlagComment) printf("\n Decomposition en %d Niveaux",NbLevels);
        
  //BOUCLE
  for(k=0;k<NbLevels;k++)
     {
      //lignes
      for(i=0;i<sz;i++) Haar1D(mat_dec[i],k);

      //colonnes
      for(j=0;j<sz;j++) 
         {
          for(l=0;l<sz;l++)       vect_dec[l]=mat_dec[l][j];
          Haar1D(vect_dec,k);
          for(l=0;l<sz;l++)       mat_dec[l][j]=vect_dec[l];
         }
      sz/=2;
     }

  //Visu
  for(i=0;i<size;i++) for(j=0;j<size;j++) mat_decvisu[i][j]=mat_dec[i][j];
  RecalImgDec(mat_decvisu);
 }


//----------------------------------------------------------
// Decompose2D
//----------------------------------------------------------
void Haar::DecomposeInvHaar2D()
 {
  int i,j,k,l;
  int sz;

  //Presentation
  if (FlagComment)
     {
      printf("\n\n\n");
      printf("\n ---------------------");
      printf("\n Reconstruction Image ");
      printf("\n ---------------------");
      printf("\n");
     }

  //Initialisation
  if (FlagComment) printf("\n Decomposition inverse en %d Niveaux",NbLevels);
  for(i=0;i<size;i++) for(j=0;j<size;j++) mat_rec[i][j]=mat_dec[i][j];
  sz=(int)(size/pow(2,(NbLevels-1)));
  if (FlagComment) printf("\n Size:%d",sz);
  
  //BOUCLE
  for(k=(NbLevels-1);k>=0;k--)
     { 
      //colonnes
      for(j=0;j<sz;j++) 
         {
          for(l=0;l<sz;l++)       vect_rec[l]=mat_rec[l][j];
          HaarInv1D(vect_rec,k);
          for(l=0;l<sz;l++)       mat_rec[l][j]=vect_rec[l];
         }

      //lignes
      for(i=0;i<sz;i++) HaarInv1D(mat_rec[i],k);

      sz*=2;
     }
 }


//-----------//
//-- UTILE --//
//-----------//
//----------------------------------------------------------
// Haar1d
//----------------------------------------------------------
void Haar::Haar1D(float* vct,int Nlev)
{
 int k,middle;
  
  //Init & Recup
  middle=(int)(size/pow(2.0,(Nlev+1)));
  for(k=0;k<size;k++) vect_tmp[k]=vct[k];
  
  //Remplissage
  for(k=0;k<middle;k++)
     {
      vct[k]=((vect_tmp[2*k]+vect_tmp[(2*k)+1])/sqrt(2.0));
      vct[k+middle]=((vect_tmp[2*k]-vect_tmp[(2*k)+1])/sqrt(2.0));
     }
}

//----------------------------------------------------------
// HaarInv1d
//----------------------------------------------------------
void Haar::HaarInv1D(float* vct,int Nlev)
{
 int k,middle;
  
  //Init & Recup
  middle=(int)(size/pow(2.0,(Nlev+1)));
  for(k=0;k<size;k++) vect_tmp[k]=vct[k];
  
  //Remplissage
  for(k=0;k<middle;k++)
     {
      vct[2*k]=((vect_tmp[k]+vect_tmp[k+middle])/sqrt(2));
      vct[(2*k)+1]=((vect_tmp[k]-vect_tmp[k+middle])/sqrt(2));
     }
}

//----------------------------------------------------------
// RecalImgDec
//----------------------------------------------------------
void Haar::RecalImgDec(float** mt)
{
 int i,j;
 float min,max;
 int minsz;

 //Initialisation
 minsz=(int)(size/pow(2,NbLevels));
 if (FlagComment) printf("\n Imagette Moyenne Taille: %d",minsz);

 //Imagette Moyenne
 min=BIGNUMBER;
 max=-BIGNUMBER;
 for(i=0;i<minsz;i++) for(j=0;j<minsz;j++)
    { if (mt[i][j]<min) min=mt[i][j];
      if (mt[i][j]>max) max=mt[i][j]; }
 if (FlagComment) printf("\n Img Moyenne  >   Min=%.1f : Max=%.1f",min,max);

 for(i=0;i<minsz;i++) for(j=0;j<minsz;j++) mt[i][j]=((mt[i][j]-min)*255.0/(max-min));

 //Imagette Coeff Ondelette
 min=BIGNUMBER;
 max=-BIGNUMBER;
 for(i=0;i<size;i++) for(j=0;j<size;j++)
    if (!((i<minsz)&&(j<minsz)))
    { if (mt[i][j]<min) min=mt[i][j];
      if (mt[i][j]>max) max=mt[i][j]; }
 if (FlagComment) printf("\n Img CoefHaar > Min=%.1f : Max=%.1f",min,max);

 for(i=0;i<size;i++) for(j=0;j<size;j++) 
    if (!((i<minsz)&&(j<minsz)))  mt[i][j]=((mt[i][j]-min)*255.0/(max-min));
}

//-----------//
//-- DEBUG --//
//-----------//
//----------------------------------------------------------
// Affiche Vecteur
//----------------------------------------------------------
void Haar::AfficheVect(float* vct,int dim)
{
 int k;

 //Boucle
 printf("\n");
 for(k=0;k<dim;k++) printf(" [%.2f]",vct[k]);
}


//------------------------------//
//------------UDWT--------------//
//------------------------------//
//------------------------------//
//----------------------------------------------------------
// UDWT_2D
//----------------------------------------------------------
void Haar::UDWT_2D(int nlev)
{
 int i,j,k,l;
 int sz;

  //Presentation
  if (FlagComment)
     {
      printf("\n\n\n");
      printf("\n -------------------------");
      printf("\n Decomposition Image UDWT ");
      printf("\n -------------------------");
      printf("\n");
     }

  //Initialisation
  NbLevels=nlev;
  sz=size;
  if (FlagComment) printf("\n Decomposition UDWT en %d Niveaux \n",NbLevels);

  //BOUCLE
  for(k=0;k<NbLevels;k++)
     {
      if (FlagComment) printf("[size:%d]",sz);
      //lignes
      for(i=0;i<sz;i++)   UDWT_1D_(mat_udwt_dec[i],k);
      sz+=size;

      //colonnes
      for(j=0;j<sz;j++) 
         {
          for(l=0;l<sz;l++)       vect_udwt_dec[l]=mat_udwt_dec[l][j];
          UDWT_1D_(vect_udwt_dec,k);
          for(l=0;l<sz;l++)       mat_udwt_dec[l][j]=vect_udwt_dec[l];
         }
     }
}

//----------------------------------------------------------
// IUDWT_2D
//----------------------------------------------------------
void Haar::IUDWT_2D()
 {
  int i,j,k,l;
  int sz;

  //Presentation
  if (FlagComment)
     {
      printf("\n\n\n");
      printf("\n ---------------------------");
      printf("\n Reconstruction Image IUDWT ");
      printf("\n ---------------------------");
      printf("\n");
     }

  //Initialisation
  if (FlagComment) printf("\n Decomposition inverse en %d Niveaux",NbLevels);

  for(i=0;i<(size*nlev_udwt);i++) for(j=0;j<(size*nlev_udwt);j++) 
  mat_udwt_rec[i][j]=mat_udwt_dec[i][j];

  sz=(int)(size*(NbLevels+1));
  if (FlagComment) printf("\n Size:%d \n",sz);
  
  //BOUCLE
  for(k=(NbLevels-1);k>=0;k--)
     { 
       if (FlagComment) printf("[size:%d]",sz);
      //colonnes
      for(j=0;j<sz;j++) 
         {
          for(l=0;l<sz;l++)       vect_udwt_rec[l]=mat_udwt_rec[l][j];
          IUDWT_1D_(vect_udwt_rec,k);
          for(l=0;l<sz;l++)       mat_udwt_rec[l][j]=vect_udwt_rec[l];
         }

      //lignes
      for(i=0;i<sz;i++) IUDWT_1D_(mat_udwt_rec[i],k);
 
      sz-=size;
     }

 //Copy
 for(i=0;i<length;i++)  for(j=0;j<width;j++) mat_rec[i][j]=mat_udwt_rec[i][j]; 

 //Debug
 //for(i=0;i<length;i++)  for(j=0;j<width;j++)
 //  if  ((mat_rec[i][j]<0)||(mat_rec[i][j]>255)) printf("[%d-%d:%.2f]",i,j,mat_rec[i][j]);
 }


//----------------------------------------------------------
// UDWT_1D_
//----------------------------------------------------------
void Haar::UDWT_1D_(float* vct,int Nlev)
{
 int k;
 int szv;

 //Initialisation
 szv=(int)(size/pow(2,Nlev));   

 //Copiage Old Difference Vector
 Copy(&vct[size],&vect_udwt[2*size],size*Nlev); 

 //Boucle
 for(k=0;k<pow(2,Nlev);k++)
   { 
    Copy(&vct[k*szv],vect_udwt_tmp,szv);   
    Haar1D_(vect_udwt_tmp,szv);      

    Copy(&vect_udwt_tmp[0],&vect_udwt[k*szv],(szv/2));
    Copy(&vect_udwt_tmp[szv/2],&vect_udwt[size+(k*szv)],(szv/2)); 

    Copy(&vct[k*szv],vect_udwt_tmp,szv);
    Decal(vect_udwt_tmp,szv); 
    Haar1D_(vect_udwt_tmp,szv); 

    Copy(&vect_udwt_tmp[0],&vect_udwt[(k*szv)+(szv/2)],(szv/2)); 
    Copy(&vect_udwt_tmp[szv/2],&vect_udwt[size+(k*szv)+(szv/2)],(szv/2));
   }

 //Copy
 Copy(vect_udwt,vct,size*(Nlev+2)); 
}

//----------------------------------------------------------
// IUDWT_1D_
//----------------------------------------------------------
void Haar::IUDWT_1D_(float* vct,int Nlev)
{
 int k;
 int szv;

  //Init
  szv=(int)(size/pow(2,Nlev)); 

  //Copiage Old Diff-Vector 
  Copy(&vct[2*size],&vect_iudwt[size],size*Nlev);  

      //Boucle
      for(k=0;k<pow(2,Nlev);k++)
         { 
          Copy(&vct[k*szv],&vect_iudwt_tmp[0],szv/2);
          Copy(&vct[size+(k*szv)],&vect_iudwt_tmp[szv/2],szv/2); 

          HaarInv1D_(vect_iudwt_tmp,szv); 
          Copy(vect_iudwt_tmp,tabvect_iudwt[0],szv); 

          Copy(&vct[(k*szv)+(szv/2)],&vect_iudwt_tmp[0],szv/2); 
          Copy(&vct[size+(k*szv)+(szv/2)],&vect_iudwt_tmp[szv/2],szv/2);

          HaarInv1D_(vect_iudwt_tmp,szv); 
          Recal(vect_iudwt_tmp,szv); 
          Copy(vect_iudwt_tmp,tabvect_iudwt[1],szv); 

          Average(vect_iudwt_tmp2,tabvect_iudwt[0],tabvect_iudwt[1],szv);
          Copy(vect_iudwt_tmp2,&vect_iudwt[k*szv],szv);
         }

  //Copy
  Copy(vect_iudwt,vct,size*(Nlev+1)); 
}

//-----------------------------------//
//------------DENOISING--------------//
//-----------------------------------//
//-----------------------------------------------------------------
// UDWT sur block 16x16 overlapping
//------------------------------------------------------------------
void Haar::FilteringUDWT16x16_(float** mt,int SzImg,float tau,float sig,int nlev,int ovl)
{
  int   i,j,k,l,m,n;
 int   x,y;
 int   SzSmallWind;
 int   posr,posc;
 int   pos;
 float temp,tmp;
 float nb;

 float*** mat3d;

 //Constantes
 SzSmallWind=16; 

 //Recup Parametre
 Tau=tau;
 Sigma2Noise=sig*sig;
 NbLevels=nlev; 
 
 //Initialisation
 if (FlagComment) printf("\n Size:%d",size);

 //Alloc
 mat3d=fmatrix_allocate_3d(SzSmallWind*SzSmallWind,SzImg,SzImg);

 //Clear
 //------
  for(k=0;k<(SzSmallWind*SzSmallWind);k++)
  for(i=0;i<SzImg;i++) for(j=0;j<SzImg;j++)   
      mat3d[k][i][j]=-1.0;

 //Boucle
 //------
 for(i=0;i<SzImg;i+=ovl) for(j=0;j<SzImg;j+=ovl) 
   {
     //printf("[%d-%d]",i,j);

     for(k=0;k<SzSmallWind;k++) for(l=0;l<SzSmallWind;l++) 
        { 
	 posr=i-(SzSmallWind/2)+k;
         posc=j-(SzSmallWind/2)+l;

         if (posr<0)           posr+=SzImg;
         if (posr>(SzImg-1))   posr-=SzImg; 
 
         if (posc<0)           posc+=SzImg;
         if (posc>(SzImg-1))   posc-=SzImg;

         mat_udwt_dec[k][l]=mt[posr][posc];
        }
    
     //-------------------------
     UDWT_2D(nlev);

     if (1) //-Hard Thresh--
        { for(m=0;m<(size*nlev_udwt);m++) for(n=0;n<(size*nlev_udwt);n++)
          if (!((m<size)&&(n<size)))
	  if (fabs(mat_udwt_dec[m][n])<Sigma2Noise)  mat_udwt_dec[m][n]=0.0; }

     if (0) //-Soft Thresh--
        {
         for(m=0;m<(size*nlev_udwt);m++) for(n=0;n<(size*nlev_udwt);n++)
         if (!((m<size)&&(n<size)))
            {
             tmp=CARRE(mat_udwt_dec[m][n])-3*Tau*Sigma2Noise;

             if (tmp<0) tmp=0.0;
             if (mat_udwt_dec[m][n]!=0.0) mat_udwt_dec[m][n]=(tmp/mat_udwt_dec[m][n]);
             } 
	}

     IUDWT_2D();
     //--------------------------
   
     x=(i%SzSmallWind);
     y=(j%SzSmallWind);
     pos=((x*SzSmallWind)+y);

     for(k=0;k<SzSmallWind;k++) for(l=0;l<SzSmallWind;l++) 
        { 
          posr=i-(SzSmallWind/2)+k;
          posc=j-(SzSmallWind/2)+l;

          if (posr<0)           posr+=SzImg;
          if (posr>(SzImg-1))   posr-=SzImg; 
 
          if (posc<0)           posc+=SzImg;
          if (posc>(SzImg-1))   posc-=SzImg;

          if  (mat3d[pos][posr][posc]!=-1) printf("!");

          mat3d[pos][posr][posc]=mat_udwt_rec[k][l]; 
         }
   }

 //Moyennage
 //---------
 for(i=0;i<SzImg;i++) for(j=0;j<SzImg;j++)
    {
     temp=0.0; nb=0.0;
     for(k=0;k<(SzSmallWind*SzSmallWind);k++)
       if (mat3d[k][i][j]>0) { nb++; temp+=mat3d[k][i][j]; }
        
     if (nb) { temp/=nb;  mt[i][j]=temp; }
    }

 //Liberation Memoire
 if (mat3d) free_fmatrix_3d(mat3d,SzSmallWind*SzSmallWind);

}

//-----------------------------------------------------------------
// 
//  ((w2)-3Tau*Sigma2Bruit)+
//  ------------------------          [ (x)+= max{x,0} ]
//            w   
//
//------------------------------------------------------------------
void Haar::FilteringUDWT(float** mt,float tau,float sig,int nlev)
{
 int i,j;
 float tmp;

 //Recup Parametre
 Tau=tau;
 Sigma2Noise=sig*sig;
 NbLevels=nlev; 

 //Initialisation
 if (FlagComment) printf("\n Size:%d",size);

 //Copie Matrice
 for(i=0;i<length;i++) for(j=0;j<width;j++) mat_udwt_dec[i][j]=mt[i][j]; 

 //Decomposition
 UDWT_2D(nlev);

 //Seuillage
 for(i=0;i<(size*nlev_udwt);i++) for(j=0;j<(size*nlev_udwt);j++)
    if (!((i<size)&&(j<size)))
    {
     tmp=CARRE(mat_udwt_dec[i][j])-3*Tau*Sigma2Noise;

     if (tmp<0) tmp=0.0;
     if (mat_udwt_dec[i][j]!=0.0) mat_udwt_dec[i][j]=(tmp/mat_udwt_dec[i][j]);
    }

 //Reconstruction
 IUDWT_2D();

 //Copy
 for(i=0;i<length;i++) for(j=0;j<width;j++) mt[i][j]=mat_udwt_rec[i][j]; 
}

//--------//
//-UTILES-//
//--------//
//----------------------------------------------------------
// Haar1d
//----------------------------------------------------------
void Haar::Haar1D_(float* vct,int nb)
{
 int k,middle;
  
  //Init & Recup
  middle=(nb/2);
  for(k=0;k<nb;k++) vect_tmp[k]=vct[k];
  
  //Remplissage
  for(k=0;k<middle;k++)
     { vct[k]=((vect_tmp[2*k]+vect_tmp[(2*k)+1])/sqrt(2.0));
       vct[k+middle]=((vect_tmp[2*k]-vect_tmp[(2*k)+1])/sqrt(2.0)); }
}

//----------------------------------------------------------
// HaarInv1d
//----------------------------------------------------------
void Haar::HaarInv1D_(float* vct,int nb)
{
 int k,middle;
  
  //Init & Recup
  middle=(nb/2);
  for(k=0;k<nb;k++) vect_tmp[k]=vct[k];
  
  //Remplissage
  for(k=0;k<middle;k++)
     { vct[2*k]=((vect_tmp[k]+vect_tmp[k+middle])/sqrt(2));
       vct[(2*k)+1]=((vect_tmp[k]-vect_tmp[k+middle])/sqrt(2));}
}

//----------------------------------------------------------
// Decal Vecteur
//----------------------------------------------------------
void Haar::Decal(float* vct,int dim)
{
 int k;
 float tmp;

 //Recup
 tmp=vct[(dim-1)];

 //Boucle
 for(k=(dim-2);k>=0;k--) vct[k+1]=vct[k];

 //last
 vct[0]=tmp;
}

//----------------------------------------------------------
// Recal Vecteur
//----------------------------------------------------------
void Haar::Recal(float* vct,int dim)
{
 int k;
 float tmp;

 //Recup
 tmp=vct[0];

 //Boucle
 for(k=0;k<(dim-1);k++) vct[k]=vct[k+1];

 //last
 vct[(dim-1)]=tmp;
}

//----------------------------------------------------------
// Copy
//----------------------------------------------------------
void Haar::Copy(float* vctin,float* vctout,int nb)
{
 int k;

 //Boucle
 for(k=0;k<nb;k++) vctout[k]=vctin[k];
}

//----------------------------------------------------------
// Average
//----------------------------------------------------------
void Haar::Average(float* vct,float* vct1,float* vct2,int nb)
{
 int k;

 //Average
 for(k=0;k<nb;k++) vct[k]=((vct1[k]+vct2[k])/2.0);
}
