//---------------------------------------------------
// module  : f_Tomograph.cc                         
// auteur  : Max Mignotte                           
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include "Tomograph.h"

//------------------------------------------------
// METHODES DE LA CLASSE Tomograph ---------------
//------------------------------------------------
//----------------------------------------------------------
// Constructeur de la classe Tomograph 
//----------------------------------------------------------
Tomograph::Tomograph(int lgth,int wdth,int nbsl) 
{ 
 //Init. Parametres
 width=wdth; 
 length=lgth;
 size=length;
 nbslices=nbsl;
 width_radon=width;
 length_radon=nbslices;

 //Verification
 if (width!=length)
    { printf("\n Image non carree");
      //exit(-1); 
    }
 
 //Allocation memoire Matrix
 mat_img=fmatrix_allocate_2d(length,width);
 mat_tmp1=fmatrix_allocate_2d(length,width);

 mat_radon=fmatrix_allocate_2d(length_radon,width_radon);
 mat_radon_visu=fmatrix_allocate_2d(length_radon,width_radon);

 mat_radonRFFT=fmatrix_allocate_2d(length_radon,width_radon);
 mat_radonIFFT=fmatrix_allocate_2d(length_radon,width_radon);
 mat_radonMFFT=fmatrix_allocate_2d(length_radon,width_radon);
 
 //Allocation memoire Vector
 VctR=fmatrix_allocate_1d(width);
 VctI=fmatrix_allocate_1d(width);
 VctM=fmatrix_allocate_1d(width);
}

//----------------------------------------------------------
// Destructeur de la classe Tomograph
//----------------------------------------------------------
Tomograph::~Tomograph()
{ 
 //Liberation memoire Mtrix
 free_fmatrix_2d(mat_img);
 free_fmatrix_2d(mat_tmp1);
 free_fmatrix_2d(mat_radon);
 free_fmatrix_2d(mat_radon_visu);
 free_fmatrix_2d(mat_radonRFFT);
 free_fmatrix_2d(mat_radonIFFT);
 free_fmatrix_2d(mat_radonMFFT);

 //Liberation memoire Vector
 free(VctR);
 free(VctI);
 free(VctM); 
}

//----------------------------------------------------------
// Simule Tomograph
//----------------------------------------------------------
void Tomograph::SimulateTomograph(int** mt)
{
 int i,j,t,r;
 int theta;
 float rotat;
 float tmp;

 //Presentation
 printf("\n\n");
 printf("\n     ------------");
 printf("\n      TOMOGRAPHE ");
 printf("\n     ------------");
 printf("\n       > Nb.Slices : [%d]\n",nbslices); 
  
 //Load Image 
 printf("\n       > Chargement Image");
 for(i=0;i<length;i++)  for(j=0;j<width;j++)  mat_img[i][j]=mt[i][j];
 CopySquareMatrix(mat_img,mat_tmp1,size);

 //Initialisation
 for(i=0;i<length_radon;i++) for(j=0;j<width_radon;j++)
    { mat_radon[i][j]=0.0;
      mat_radon_visu[i][j]=0.0;
      mat_radonRFFT[i][j]=0.0;
      mat_radonIFFT[i][j]=0.0;
      mat_radonMFFT[i][j]=0.0; }

  for(i=0;i<width;i++) 
     { VctR[i]=0.0;
       VctI[i]=0.0; 
       VctM[i]=0.0; }

 
   //Calcule Transformee Radon 
   printf("\n       > Transformee Radon");
   printf("\n          ");
   for(theta=0;theta<nbslices;theta++)
     {
      //Information
       if ((theta%20)==0) printf("[%d]",theta);
       fflush(stdout);

      //Rotation Img
      rotat=(float)theta*PI/(float)nbslices;          //Ajout Float
      rotation(mat_tmp1,mat_img,length,width,-rotat);
       
      //Addition ligneProjection
      for (t=0;t<length;t++) for (j=0;j<width;j++) 
          mat_radon[theta][t]+=mat_tmp1[t][j];
     }

   //Pour Visu
   printf("\n       > Recalage Image");
   for(i=0;i<length_radon;i++)  for(j=0;j<width_radon;j++)
     mat_radon_visu[i][j]=mat_radon[i][j];

   Recal(mat_radon_visu,length_radon,width_radon);  

   //Estimation MoyenneImage
   tmp=0.0;
   for(i=0;i<length_radon;i++) for(j=0;j<width_radon;j++) tmp+=mat_radon[i][j];
   tmp/=(length_radon*width_radon);
   tmp/=(size);
   MoyImg=tmp;
   printf("\n       > MoyImg (Estime /TF-Radon)=%.2f\n",MoyImg);

   //Transformee Fourier lignes par lignes
   printf("       > FFT Transf. Radon");
    for (theta=0;theta<nbslices;theta++)
     {
       for (t=0;t<width;t++) 
	 {
	   VctR[t]=mat_radon[theta][t]; 
	   VctI[t]=0.0;
	 }

       ReMkeVct(VctR,width); 

       //-TFD------------------------------------------- 
       if (IsPowerOfTwo(width))  FFT1D(VctR,VctI,width);
       if (!IsPowerOfTwo(width)) DFT(VctR,VctI,width,1);
       //-----------------------------------------------

       ReMkeVct(VctR,width);
       ReMkeVct(VctI,width);
       ModVct(VctM,VctR,VctI,width); 

       for (r=0;r<width;r++) 
	 {
	  mat_radonRFFT[theta][r]=VctR[r];
	  mat_radonIFFT[theta][r]=VctI[r];
	  mat_radonMFFT[theta][r]=VctM[r];
	 }
     }

    //Visualisation
   Recal(mat_radonMFFT,length_radon,width_radon);
   Mult(mat_radonMFFT,30.0,length_radon,width_radon);      
}

//----------------------------------------------------------
// ReComputeFFT
//----------------------------------------------------------
void Tomograph::ReComputeFFT()
{
 int t,r;
 int theta;

 //Transformee Fourier lignes par lignes
 printf("       > FFT Transf. Radon [NbSlices=%d]",nbslices);
    for (theta=0;theta<nbslices;theta++)
     {
       for (t=0;t<width;t++) 
	 {
	   VctR[t]=mat_radon[theta][t]; 
	   VctI[t]=0.0;
	 }

       ReMkeVct(VctR,width);
    
       //-TFD------------------------------------------
       if (IsPowerOfTwo(width))  FFT1D(VctR,VctI,width);
       if (!IsPowerOfTwo(width)) DFT(VctR,VctI,width,1); 
       //----------------------------------------------
  
       ReMkeVct(VctR,width);
       ReMkeVct(VctI,width);
       ModVct(VctM,VctR,VctI,width); 

       for (r=0;r<width;r++) 
	 {
	  mat_radonRFFT[theta][r]=VctR[r];
	  mat_radonIFFT[theta][r]=VctI[r];
	  mat_radonMFFT[theta][r]=VctM[r];
	 }
     }

    //Visualisation
   Recal(mat_radonMFFT,length_radon,width_radon);
   Mult(mat_radonMFFT,30.0,length_radon,width_radon); 
}

//-----------//
//-- UTILE --//
//-----------//
//----------------------------------------------------------
// Rotation
//----------------------------------------------------------
void Tomograph::rotation(float** mat2,float** mat1,int lgth,int wdth,float angle)
{
 int i, j, x, y;
 float xprime, yprime, tmp1, tmp2;

 //Rotation
  for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)
     {
      xprime=cos(angle)*(i-lgth/2)+sin(angle)*(j-wdth/2)+lgth/2;
      yprime=-sin(angle)*(i-lgth/2)+cos(angle)*(j-wdth/2)+wdth/2;
      x=(int) floor(xprime);
      y=(int) floor(yprime);
      if ((x>=0)&&(x<lgth-1)&&(y>=0)&&(y<wdth-1))
         {
         tmp1=mat1[x][y]+(xprime-x)*(mat1[x+1][y]-mat1[x][y]);
         tmp2=mat1[x][y+1]+(xprime-x)*(mat1[x+1][y+1]-mat1[x][y+1]);
         mat2[i][j]=tmp1+(yprime-y)*(tmp2-tmp1);
         }
     else mat2[i][j]=0.0;
    }
}


//----------------------------------------------------------
// ComputeMoyImg
//----------------------------------------------------------
void Tomograph::ComputeMoyImg()
{
 float tmp;
 int i,j;

 tmp=0.0;
 //Compute Moy
 for(i=0;i<length_radon;i++) for(j=0;j<width_radon;j++) tmp+=mat_radon[i][j];
 tmp/=(length_radon*width_radon);
 tmp/=(size);
 MoyImg=tmp;
 printf("\n       > MoyImg (Estime /TF-Radon)=%.2f\n",MoyImg);
}

//----------------------------------------------------------
// Addition
//----------------------------------------------------------
void Tomograph::Addition()
{
 int i,k;
 int cpt;
 float min,max;

 //Affichage
 printf("\n       >>>> Addition Sinogram >>>> \n");

 //Compute Addition
 for(cpt=0,k=(int)(nbslices/2.0);k<=(int)(nbslices-1);k++,cpt++)
    {
     //printf("[%d>%d] ",cpt,k);
     for(i=0;i<width_radon;i++)
        { mat_radon[cpt][i]+=mat_radon[k][width_radon-1-i];
          mat_radon_visu[cpt][i]+=mat_radon_visu[k][width_radon-1-i]; }
    }

 //NbSlices
 nbslices=(int)((float)nbslices/2.0);
 printf("\n       >Nouveau Nombre de Slices >>>> [%d] ",nbslices);

 //Recalage mat_radon_visu
 min=1000000;
 max=-1000000;
 for(k=0;k<nbslices;k++) for(i=0;i<width_radon;i++)
   { if (min>mat_radon_visu[k][i]) min=mat_radon_visu[k][i];
     if (max<mat_radon_visu[k][i]) max=mat_radon_visu[k][i]; }

 for(k=0;k<nbslices;k++) for(i=0;i<width_radon;i++)
   mat_radon_visu[k][i]=(((mat_radon_visu[k][i]-min)/(max-min))*255.0);

 min=1000000;
 max=-1000000;
 for(k=0;k<nbslices;k++) for(i=0;i<width_radon;i++)
   { if (min>mat_radon_visu[k][i]) min=mat_radon_visu[k][i];
     if (max<mat_radon_visu[k][i]) max=mat_radon_visu[k][i]; }

 printf("\n       >[min:%.1f>max:%1.f]",min,max);
}

//----------------------------------------------------------
// 
//----------------------------------------------------------

 
